#include <stdlib.h>
#include <new>

#include "pc6001v.h"
#include "config.h"
#include "error.h"
#include "osd.h"

const P6KeyName CFG6::P6KeyNameDef[] = {	// P6L[R[h ̒`
	{ KP6_UNKNOWN,		"K6_UNKNOWN" },
	
	{ KP6_1,			"K6_1" },
	{ KP6_2,			"K6_2" },
	{ KP6_3,			"K6_3" },
	{ KP6_4,			"K6_4" },
	{ KP6_5,			"K6_5" },
	{ KP6_6,			"K6_6" },
	{ KP6_7,			"K6_7" },
	{ KP6_8,			"K6_8" },
	{ KP6_9,			"K6_9" },
	{ KP6_0,			"K6_0" },
	
	{ KP6_A,			"K6_A" },
	{ KP6_B,			"K6_B" },
	{ KP6_C,			"K6_C" },
	{ KP6_D,			"K6_D" },
	{ KP6_E,			"K6_E" },
	{ KP6_F,			"K6_F" },
	{ KP6_G,			"K6_G" },
	{ KP6_H,			"K6_H" },
	{ KP6_I,			"K6_I" },
	{ KP6_J,			"K6_J" },
	{ KP6_K,			"K6_K" },
	{ KP6_L,			"K6_L" },
	{ KP6_M,			"K6_M" },
	{ KP6_N,			"K6_N" },
	{ KP6_O,			"K6_O" },
	{ KP6_P,			"K6_P" },
	{ KP6_Q,			"K6_Q" },
	{ KP6_R,			"K6_R" },
	{ KP6_S,			"K6_S" },
	{ KP6_T,			"K6_T" },
	{ KP6_U,			"K6_U" },
	{ KP6_V,			"K6_V" },
	{ KP6_W,			"K6_W" },
	{ KP6_X,			"K6_X" },
	{ KP6_Y,			"K6_Y" },
	{ KP6_Z,			"K6_Z" },
	
	{ KP6_F1,			"K6_F1" },
	{ KP6_F2,			"K6_F2" },
	{ KP6_F3,			"K6_F3" },
	{ KP6_F4,			"K6_F4" },
	{ KP6_F5,			"K6_F5" },
	
	{ KP6_MINUS,		"K6_MINUS" },
	{ KP6_CARET,		"K6_CARET" },
	{ KP6_YEN,			"K6_YEN" },
	{ KP6_AT,			"K6_AT" },
	{ KP6_LBRACKET,		"K6_LBRACKET" },
	{ KP6_RBRACKET,		"K6_RBRACKET" },
	{ KP6_SEMICOLON,	"K6_SEMICOLON" },
	{ KP6_COLON,		"K6_COLON" },
	{ KP6_COMMA,		"K6_COMMA" },
	{ KP6_PERIOD,		"K6_PERIOD" },
	{ KP6_SLASH,		"K6_SLASH" },
	{ KP6_UNDERSCORE,	"K6_UNDERSCORE" },
	{ KP6_SPACE,		"K6_SPACE" },
	
	{ KP6_ESC,			"K6_ESC" },
	{ KP6_TAB,			"K6_TAB" },
	{ KP6_CTRL,			"K6_CTRL" },
	{ KP6_SHIFT,		"K6_SHIFT" },
	{ KP6_GRAPH,		"K6_GRAPH" },
	{ KP6_HOME,			"K6_HOME" },
	{ KP6_STOP,			"K6_STOP" },
	{ KP6_PAGE,			"K6_PAGE" },
	{ KP6_RETURN,		"K6_RETURN" },
	{ KP6_KANA,			"K6_KANA" },
	{ KP6_INS,			"K6_INS" },
	{ KP6_DEL,			"K6_DEL" },
	
	{ KP6_UP,			"K6_UP" },
	{ KP6_DOWN,			"K6_DOWN" },
	{ KP6_LEFT,			"K6_LEFT" },
	{ KP6_RIGHT,		"K6_RIGHT" },
	
	{ KP6_MODE,			"K6_MODE" },
	{ KP6_CAPS,			"K6_CAPS" },
	
	
	// eL[g
	{ KP6_P0,			"K6_P0" },
	{ KP6_P1,			"K6_P1" },
	{ KP6_P2,			"K6_P2" },
	{ KP6_P3,			"K6_P3" },
	{ KP6_P4,			"K6_P4" },
	{ KP6_P5,			"K6_P5" },
	{ KP6_P6,			"K6_P6" },
	{ KP6_P7,			"K6_P7" },
	{ KP6_P8,			"K6_P8" },
	{ KP6_P9,			"K6_P9" },
	{ KP6_PPLUS,		"K6_PPLUS" },
	{ KP6_PMINUS,		"K6_PMINUS" },
	{ KP6_PMULTIPLY,	"K6_PMULTIPLY" },
	{ KP6_PDIVIDE,		"K6_PDIVIDE" },
	{ KP6_PPERIOD,		"K6_PPERIOD" },
	{ KP6_PRETURN,		"K6_PRETURN" },
	
	
	// e@\L[
	{ KFN_1,			"K6_FN1" },
	{ KFN_2,			"K6_FN2" },
	{ KFN_3,			"K6_FN3" },
	{ KFN_4,			"K6_FN4" },
	{ KFN_5,			"K6_FN5" },
	{ KFN_6,			"K6_FN6" },
	{ KFN_7,			"K6_FN7" },
	{ KFN_8,			"K6_FN8" },
	{ KFN_9,			"K6_FN9" }
};


const PCKeyName CFG6::PCKeyNameDef[] = {	// zL[R[h ̒`
	{ KVC_UNKNOWN,		"K_UNKNOWN" },
	
	{ KVC_1,			"K_1" },
	{ KVC_2,			"K_2" },
	{ KVC_3,			"K_3" },
	{ KVC_4,			"K_4" },
	{ KVC_5,			"K_5" },
	{ KVC_6,			"K_6" },
	{ KVC_7,			"K_7" },
	{ KVC_8,			"K_8" },
	{ KVC_9,			"K_9" },
	{ KVC_0,			"K_0" },
	
	{ KVC_A,			"K_A" },
	{ KVC_B,			"K_B" },
	{ KVC_C,			"K_C" },
	{ KVC_D,			"K_D" },
	{ KVC_E,			"K_E" },
	{ KVC_F,			"K_F" },
	{ KVC_G,			"K_G" },
	{ KVC_H,			"K_H" },
	{ KVC_I,			"K_I" },
	{ KVC_J,			"K_J" },
	{ KVC_K,			"K_K" },
	{ KVC_L,			"K_L" },
	{ KVC_M,			"K_M" },
	{ KVC_N,			"K_N" },
	{ KVC_O,			"K_O" },
	{ KVC_P,			"K_P" },
	{ KVC_Q,			"K_Q" },
	{ KVC_R,			"K_R" },
	{ KVC_S,			"K_S" },
	{ KVC_T,			"K_T" },
	{ KVC_U,			"K_U" },
	{ KVC_V,			"K_V" },
	{ KVC_W,			"K_W" },
	{ KVC_X,			"K_X" },
	{ KVC_Y,			"K_Y" },
	{ KVC_Z,			"K_Z" },
	
	{ KVC_F1,			"K_F1" },
	{ KVC_F2,			"K_F2" },
	{ KVC_F3,			"K_F3" },
	{ KVC_F4,			"K_F4" },
	{ KVC_F5,			"K_F5" },
	{ KVC_F6,			"K_F6" },
	{ KVC_F7,			"K_F7" },
	{ KVC_F8,			"K_F8" },
	{ KVC_F9,			"K_F9" },
	{ KVC_F10,			"K_F10" },
	{ KVC_F11,			"K_F11" },
	{ KVC_F12,			"K_F12" },
	
	{ KVC_MINUS,		"K_MINUS" },
	{ KVC_CARET,		"K_CARET" },
	{ KVC_BACKSPACE,	"K_BACKSPACE" },
	{ KVC_AT,			"K_AT" },
	{ KVC_LBRACKET,		"K_LBRACKET" },
	{ KVC_SEMICOLON,	"K_SEMICOLON" },
	{ KVC_COLON,		"K_COLON" },
	{ KVC_COMMA,		"K_COMMA" },
	{ KVC_PERIOD,		"K_PERIOD" },
	{ KVC_SLASH,		"K_SLASH" },
	{ KVC_SPACE,		"K_SPACE" },
	
	{ KVC_ESC,			"K_ESC" },
	{ KVC_HANZEN,		"K_HANZEN" },
	{ KVC_TAB,			"K_TAB" },
	{ KVC_CAPSLOCK,		"K_CAPSLOCK" },
	{ KVC_ENTER,		"K_ENTER" },
	{ KVC_LCTRL,		"K_LCTRL" },
	{ KVC_RCTRL,		"K_RCTRL" },
	{ KVC_LSHIFT,		"K_LSHIFT" },
	{ KVC_RSHIFT,		"K_RSHIFT" },
	{ KVC_LALT,			"K_LALT" },
	{ KVC_RALT,			"K_RALT" },
	{ KVC_PRINT,		"K_PRINT" },
	{ KVC_SCROLLLOCK,	"K_SCROLLLOCK" },
	{ KVC_PAUSE,		"K_PAUSE" },
	{ KVC_INSERT,		"K_INSERT" },
	{ KVC_DELETE,		"K_DELETE" },
	{ KVC_HOME,			"K_HOME" },
	{ KVC_END,			"K_END" },
	{ KVC_PAGEUP,		"K_PAGEUP" },
	{ KVC_PAGEDOWN,		"K_PAGEDOWN" },
	
	{ KVC_UP,			"K_UP" },
	{ KVC_DOWN,			"K_DOWN" },
	{ KVC_LEFT,			"K_LEFT" },
	{ KVC_RIGHT,		"K_RIGHT" },
	
	{ KVC_P0,			"K_P_0" },
	{ KVC_P1,			"K_P_1" },
	{ KVC_P2,			"K_P_2" },
	{ KVC_P3,			"K_P_3" },
	{ KVC_P4,			"K_P_4" },
	{ KVC_P5,			"K_P_5" },
	{ KVC_P6,			"K_P_6" },
	{ KVC_P7,			"K_P_7" },
	{ KVC_P8,			"K_P_8" },
	{ KVC_P9,			"K_P_9" },
	{ KVC_NUMLOCK,		"K_NUMLOCK" },
	{ KVC_P_PLUS,		"K_P_PLUS" },
	{ KVC_P_MINUS,		"K_P_MINUS" },
	{ KVC_P_MULTIPLY,	"K_P_MULTIPLY" },
	{ KVC_P_DIVIDE,		"K_P_DIVIDE" },
	{ KVC_P_PERIOD,		"K_P_PERIOD" },
	{ KVC_P_ENTER,		"K_P_ENTER" },
	
	// {L[{[ĥ
	{ KVC_YEN,			"K_YEN" },
	{ KVC_RBRACKET,		"K_RBRACKET" },
	{ KVC_UNDERSCORE,	"K_UNDERSCORE" },
	{ KVC_MUHENKAN,		"K_MUHENKAN" },
	{ KVC_HENKAN,		"K_HENKAN" },
	{ KVC_HIRAGANA,		"K_HIRAGANA" },
	
	// pL[{[ĥ
	{ KVE_BACKSLASH,	"K_BACKSLASH" },
	
	// ǉL[
	{ KVX_RMETA,		"K_RMETA" },
	{ KVX_LMETA,		"K_LMETA" },
	{ KVX_MENU,			"K_MENU" },
};


const VKeyConv CFG6::KeyIni[] = {	// zL[R[h -> P6L[R[h`l
	{ KVC_1,			KP6_1 },			// 1	!
	{ KVC_2,			KP6_2 },			// 2	"
	{ KVC_3,			KP6_3 },			// 3	#
	{ KVC_4,			KP6_4 },			// 4	$
	{ KVC_5,			KP6_5 },			// 5	%
	{ KVC_6,			KP6_6 },			// 6	&
	{ KVC_7,			KP6_7 },			// 7	'
	{ KVC_8,			KP6_8 },			// 8	(
	{ KVC_9,			KP6_9 },			// 9	)
	{ KVC_0,			KP6_0 },			// 0
	
	{ KVC_A,			KP6_A },			// a	A
	{ KVC_B,			KP6_B },			// b	B
	{ KVC_C,			KP6_C },			// c	C
	{ KVC_D,			KP6_D },			// d	D
	{ KVC_E,			KP6_E },			// e	E
	{ KVC_F,			KP6_F },			// f	F
	{ KVC_G,			KP6_G },			// g	G
	{ KVC_H,			KP6_H },			// h	H
	{ KVC_I,			KP6_I },			// i	I
	{ KVC_J,			KP6_J },			// j	J
	{ KVC_K,			KP6_K },			// k	K
	{ KVC_L,			KP6_L },			// l	L
	{ KVC_M,			KP6_M },			// m	M
	{ KVC_N,			KP6_N },			// n	N
	{ KVC_O,			KP6_O },			// o	O
	{ KVC_P,			KP6_P },			// p	P
	{ KVC_Q,			KP6_Q },			// q	Q
	{ KVC_R,			KP6_R },			// r	R
	{ KVC_S,			KP6_S },			// s	S
	{ KVC_T,			KP6_T },			// t	T
	{ KVC_U,			KP6_U },			// u	U
	{ KVC_V,			KP6_V },			// v	V
	{ KVC_W,			KP6_W },			// w	W
	{ KVC_X,			KP6_X },			// x	X
	{ KVC_Y,			KP6_Y },			// y	Y
	{ KVC_Z,			KP6_Z },			// z	Z
	
	{ KVC_F1,			KP6_F1 },			// F1
	{ KVC_F2,			KP6_F2 },			// F2
	{ KVC_F3,			KP6_F3 },			// F3
	{ KVC_F4,			KP6_F4 },			// F4
	{ KVC_F5,			KP6_F5 },			// F5
	
	{ KVC_MINUS,		KP6_MINUS },		// -	=
	{ KVC_CARET,		KP6_CARET },		// ^	~
	{ KVC_BACKSPACE,	KP6_DEL },			// BackSpace
	{ KVC_AT,			KP6_AT },			// @	`
	{ KVC_LBRACKET,		KP6_LBRACKET },		// [	{
	{ KVC_SEMICOLON,	KP6_SEMICOLON },	// ;	+
	{ KVC_COLON,		KP6_COLON },		// :	*
	{ KVC_COMMA,		KP6_COMMA },		// ,	<
	{ KVC_PERIOD,		KP6_PERIOD },		// .	>
	{ KVC_SLASH,		KP6_SLASH },		// /	?
	{ KVC_SPACE,		KP6_SPACE },		// Space
	
	{ KVC_ESC,			KP6_ESC },			// ESC
	{ KVC_HANZEN,		KP6_UNKNOWN },		// p/Sp
	{ KVC_TAB,			KP6_TAB },			// Tab
	{ KVC_CAPSLOCK,		KP6_UNKNOWN },		// CapsLock
	{ KVC_ENTER,		KP6_RETURN },		// Enter
	{ KVC_LCTRL,		KP6_CTRL },			// L-Ctrl
	{ KVC_RCTRL,		KP6_CTRL },			// R-Ctrl
	{ KVC_LSHIFT,		KP6_SHIFT },		// L-Shift
	{ KVC_RSHIFT,		KP6_SHIFT },		// R-Shift
	{ KVC_LALT,			KP6_GRAPH },		// L-Alt
	{ KVC_RALT,			KP6_GRAPH },		// R-Alt
	{ KVC_PRINT,		KP6_UNKNOWN },		// PrintScreen
	{ KVC_SCROLLLOCK,	KP6_CAPS },			// ScrollLock
	{ KVC_PAUSE,		KP6_KANA },			// Pause
	{ KVC_INSERT,		KP6_INS },			// Insert
	{ KVC_DELETE,		KP6_DEL },			// Delete
	{ KVC_END,			KP6_STOP },			// End
	{ KVC_HOME,			KP6_HOME },			// Home
	{ KVC_PAGEUP,		KP6_PAGE },			// PageUp
	{ KVC_PAGEDOWN,		KP6_MODE },			// PageDown
	
	{ KVC_UP,			KP6_UP },			// 
	{ KVC_DOWN,			KP6_DOWN },			// 
	{ KVC_LEFT,			KP6_LEFT },			// 
	{ KVC_RIGHT,		KP6_RIGHT },		// 
	
	{ KVC_P0,			KP6_P0 },			// [0]
	{ KVC_P1,			KP6_P1 },			// [1]
	{ KVC_P2,			KP6_P2 },			// [2]
	{ KVC_P3,			KP6_P3 },			// [3]
	{ KVC_P4,			KP6_P4 },			// [4]
	{ KVC_P5,			KP6_P5 },			// [5]
	{ KVC_P6,			KP6_P6 },			// [6]
	{ KVC_P7,			KP6_P7 },			// [7]
	{ KVC_P8,			KP6_P8 },			// [8]
	{ KVC_P9,			KP6_P9 },			// [9]
	{ KVC_NUMLOCK,		KP6_UNKNOWN },		// NumLock
	{ KVC_P_PLUS,		KP6_PPLUS },		// [+]
	{ KVC_P_MINUS,		KP6_PMINUS },		// [-]
	{ KVC_P_MULTIPLY,	KP6_PMULTIPLY },	// [*]
	{ KVC_P_DIVIDE,		KP6_PDIVIDE },		// [/]
	{ KVC_P_PERIOD,		KP6_PPERIOD },		// [.]
	{ KVC_P_ENTER,		KP6_PRETURN },		// [Enter]
	
	// {L[{[ĥ
	{ KVC_YEN,			KP6_YEN },			// 	|
	{ KVC_RBRACKET,		KP6_RBRACKET },		// ]	}
	{ KVC_UNDERSCORE,	KP6_UNDERSCORE },	// \	_
//	{ KVC_MUHENKAN,		KP6_UNKNOWN },		// ϊ
//	{ KVC_HENKAN,		KP6_UNKNOWN },		// ϊ
//	{ KVC_HIRAGANA,		KP6_UNKNOWN },		// Ђ炪
	
	// pL[{[ĥ
	{ KVE_BACKSLASH,	KP6_YEN	 },			// 			BackSlash	|
	
	// ǉL[
	{ KVX_RMETA,		KP6_UNKNOWN },		// L-Meta
	{ KVX_LMETA,		KP6_UNKNOWN },		// R-Meta
	{ KVX_MENU,			KP6_UNKNOWN },		// Menu
	
	// e@\L[ (̂Ƃ떳)
//	{ KVC_F6,			KFN_1 },			// F6
//	{ KVC_F7,			KFN_2 },			// F7
//	{ KVC_F8,			KFN_3 },			// F8
//	{ KVC_F9,			KFN_4 },			// F9
//	{ KVC_F10,			KFN_5 },			// F10
//	{ KVC_F11,			KFN_6 },			// F11
//	{ KVC_F12,			KFN_7 },			// F12
	
	{ KVC_LAST,			KP6_LAST }
};


const COLOR24 CFG6::STDColor[] = {	// WJ[f[^ ( R,G,B,0  0-255 )
				// VXeJ[
				{   0,   0,   0, 0 },	// 00:
				{   0,   0, 128, 0 },	// 01:
				{   0, 128,   0, 0 },	// 02:
				{   0, 128, 128, 0 },	// 03:
				{ 128,   0,   0, 0 },	// 04:
				{ 128,   0, 128, 0 },	// 05:
				{ 128, 128,   0, 0 },	// 06:
				{ 128, 128, 128, 0 },	// 07:
				{ 192, 192, 192, 0 },	// 08:
				{   0,   0, 255, 0 },	// 09:
				{   0, 255,   0, 0 },	// 10:
				{   0, 255, 255, 0 },	// 11:
				{ 255,   0,   0, 0 },	// 12:
				{ 255,   0, 255, 0 },	// 13:
				{ 255, 255,   0, 0 },	// 14:
				{ 255, 255, 255, 0 },	// 15:
				
				{  30,  30,  30, 0 },	// 16:(mode 1,2 {[_[)
				
				// mode 1
				{  50, 238,  15, 0 },	// 17:(Set1)
				{   4, 106,   4, 0 },	// 18:[
				{ 252, 100,  35, 0 },	// 19:(Set2)
				{ 180,   6,   4, 0 },	// 20:[
				// mode 2
				{  50, 238,  15, 0 },	// 21:
				{ 234, 224,   4, 0 },	// 22:
				{  68,  10, 244, 0 },	// 23:
				{ 226,   6,  12, 0 },	// 24:
				{ 214, 208, 246, 0 },	// 25:
				{  52, 186, 236, 0 },	// 26:VA
				{ 244,  10, 244, 0 },	// 27:}[^
				{ 252,  90,   4, 0 },	// 28:
				// mode 3
				{  50, 238,  15, 0 },	// 29:
				{ 234, 224,   4, 0 },	// 30:
				{  68,  10, 244, 0 },	// 31:
				{ 226,   6,  12, 0 },	// 32:
				{ 214, 208, 246, 0 },	// 33:
				{  52, 186, 236, 0 },	// 34:VA
				{ 244,  10, 244, 0 },	// 35:}[^
				{ 252,  90,   4, 0 },	// 36:
				// mode 4
				{   4, 106,   4, 0 },	// 37:[(Set1)
				{  50, 238,  15, 0 },	// 38:
				{  30,  30,  30, 0 },	// 39:(Set2)
				{ 224, 255, 208, 0 },	// 40:
				
				{ 225,  47,  32, 0 },	// 41:ɂ (Set1)
				{ 242, 231,   9, 0 },	// 42:ɂ (Set1)
				{   0,   0,   0, 0 },	// 43:ɂ - (Set1)
				{   0,   0,   0, 0 },	// 44:ɂ - (Set1)
				{ 255,  48,   0, 0 },	// 45:ɂ (Set2)
				{ 120, 220, 255, 0 },	// 46:ɂ (Set2)
				{ 255,  72,  72, 0 },	// 47:ɂ (Set2)
				{  40, 255,  80, 0 },	// 48:ɂ (Set2)
				
				// mk2
				{  20,  20,  20, 0 },	// 49:()
				{ 255, 172,   0, 0 },	// 50:
				{   0, 255, 172, 0 },	// 51:
				{ 172, 255,   0, 0 },	// 52:
				{ 172,   0, 255, 0 },	// 53:
				{ 255,   0, 172, 0 },	// 54:Ԏ
				{   0, 172, 255, 0 },	// 55:F
				{ 172, 172, 172, 0 },	// 56:DF
				{  20,  20,  20, 0 },	// 57:
				{ 255,   0,   0, 0 },	// 58:
				{   0, 255,   0, 0 },	// 59:
				{ 255, 255,   0, 0 },	// 60:
				{   0,   0, 255, 0 },	// 61:
				{ 255,   0, 255, 0 },	// 62:}[^
				{   0, 255, 255, 0 },	// 63:VA
				{ 255, 255, 255, 0 },	// 64:
				
				// mode 4 ɂݕ␳p
				{   0,   0,   0, 0 },	// 65:ۂ
				{   0,   0,   0, 0 },	// 66:ۂ
				{   0,   0,   0, 0 },	// 67:Ԃۂ
				{ 255, 255, 255, 0 },	// 68:ۂ
				{ 255, 255, 255, 0 },	// 69:ۂ
				{ 255, 255, 255, 0 },	// 70:Ԃۂ
				{ 120, 220, 255, 0 },	// 71:ۂ
				{ 120, 220, 255, 0 },	// 72:ۂ
				{ 120, 220, 255, 0 },	// 73:Ԃۂ
				{ 255,  48, 255, 0 },	// 74:ۂ
				{ 255,  48, 255, 0 },	// 75:ۂ
				{ 255,  48, 255, 0 }	// 76:ۂ
			};




////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
CFG6::CFG6( void ) : Ini(NULL)
	#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	, MonDisp(false)
	#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
{
	// INIt@C̃pXݒ
	sprintf( IniPath, "%s" CONF_FILE, OSD_GetModulePath() );
	
	INITARRAY( Caption, '\0' );		// EBhELvV
	INITARRAY( DokoFile, '\0' );	// ǂłSAVEt@CpX
	INITARRAY( RomPath, '\0' );		// ROMpX
	INITARRAY( ExtRomPath, '\0' );	// gROMpX
	INITARRAY( ExtRomFile, '\0' );	// gROMt@C
	INITARRAY( TapePath, '\0' );	// TAPEpX
	INITARRAY( TapeFile, '\0' );	// TAPEt@C
	INITARRAY( SaveFile, '\0' );	// TAPE(SAVE)t@C
	INITARRAY( DiskPath, '\0' );	// DISKpX
	INITARRAY( DiskFile1, '\0' );	// DISK1t@C
	INITARRAY( DiskFile2, '\0' );	// DISK2t@C
	INITARRAY( ImgPath, '\0' );		// XN[Vbgi[pX
	INITARRAY( WavePath, '\0' );	// WAVEi[pX
	INITARRAY( PrinterFile, '\0' );	// v^t@C
}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
CFG6::~CFG6( void )
{
	if( Ini ){
		if( GetSaveQuit() ) Ini->Write();	// INIt@Cɏ
		delete Ini;
	}
}


////////////////////////////////////////////////////////////////
// (INIt@CǍ)
//
// :	Ȃ
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool CFG6::Init( void )
{
	// INIIuWFNg݂ĂUJ
	if( Ini ) delete Ini;
	
	try{
		// INIt@CȂȂftHgō쐬
		if( !OSD_FileExist( IniPath ) ){
			FILE *fp = FOPENEN( IniPath, "wt" );
			if( !fp ) throw Error::IniWriteFailed;
			// ^Cgso͂ĈU
			fprintf( fp, MSINI_TITLE );
			fclose( fp );
			
			// INIt@CJ
			Ini = new cIni();
			if( !Ini->Init( IniPath ) ) throw Error::IniDefault;
			InitIni( Ini, true );	// INIIuWFNglݒ
			Ini->Write();
		}else{
			// INIt@CJ
			Ini = new cIni();
			if( !Ini->Init( IniPath ) ) throw Error::IniDefault;
			InitIni( Ini, false );	// INIIuWFNglݒ
		}
	}
	catch( std::bad_alloc ){	// new Ɏsꍇ
		Error::SetError( Error::MemAllocFailed );
		return false;
	}
	catch( Error::Errno i ){	// O
		Error::SetError( i );
		if( Ini ) delete Ini;
		Ini = NULL;
		return false;
	}
	
	return true;
}


////////////////////////////////////////////////////////////////
// INIt@C
//
// :	Ȃ
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool CFG6::Write( void )
{
	if( Ini ){
		// INIt@Cɏ
		return Ini->Write();
	}
	return false;
}


////////////////////////////////////////////////////////////////
// oANZX֐
////////////////////////////////////////////////////////////////
// @擾
int CFG6::GetModel( void )
{
	int st = DEFAULT_MODEL;
	Ini->GetInt( "CONFIG", "Model", &st, st );
	return st;
}

// @ݒ
void CFG6::SetModel( int data )
{
	Ini->PutEntry( "CONFIG", MSINI_Model, "Model", "%02d", data );
}

// EBhELvV擾
char *CFG6::GetCaption( void )
{
	switch( GetModel() ){	// @擾
	case 62: sprintf( Caption, APPNAME " (" P62NAME ") Ver." VERSION ); break;
	case 66: sprintf( Caption, APPNAME " (" P66NAME ") Ver." VERSION ); break;
	case 64: sprintf( Caption, APPNAME " (" P64NAME ") Ver." VERSION ); break;
	case 68: sprintf( Caption, APPNAME " (" P68NAME ") Ver." VERSION ); break;
	default: sprintf( Caption, APPNAME " (" P60NAME ") Ver." VERSION );
	}
	return Caption;
}

// ǂłSAVEt@C擾
char *CFG6::GetDokoFile( void )
{
	return DokoFile;
}

// ǂłSAVEt@Cݒ
void CFG6::SetDokoFile( const char *str )
{
	strncpy( DokoFile, str, PATH_MAX );
	UnDelimiter( DokoFile );
	DelDelimiter( DokoFile );
}

// I[o[NbN擾
int CFG6::GetOverClock( void )
{
	int st = 100;
	Ini->GetInt( "CONFIG", "OverClock", &st, st );
	return st;
}

// I[o[NbNݒ
void CFG6::SetOverClock( int data )
{
	Ini->PutEntry( "CONFIG", MSINI_OverClock, "OverClock", "%d", data );
}

// ROMpX擾
char *CFG6::GetRomPath( void )
{
	Ini->GetString( "PATH", "RomPath", RomPath, RomPath );
	Delimiter( RomPath );
	AddDelimiter( RomPath );
	return RomPath;
}

// ROMpXݒ
void CFG6::SetRomPath( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "PATH", MSINI_RomPath, "RomPath", temp );
}

// gRAMg擾
bool CFG6::GetUseExtRam( void )
{
	bool st = DEFAULT_EXTRAM;
	Ini->GetTruth( "CONFIG", "ExtRam", &st, st );
	return st;
}

// gRAMgݒ
void CFG6::SetUseExtRam( bool yn )
{
	Ini->PutEntry( "CONFIG", MSINI_ExtRam, "ExtRam", "%s", yn ? "Yes" : "No" );
}

// gROMpX擾
char *CFG6::GetExtRomPath( void )
{
	Ini->GetString( "PATH", "ExtRomPath", ExtRomPath, ExtRomPath );
	Delimiter( ExtRomPath );
	AddDelimiter( ExtRomPath );
	return ExtRomPath;
}
// gROMpXݒ
void CFG6::SetExtRomPath( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "PATH", MSINI_ExtRomPath, "ExtRomPath", temp );
}

// gROMt@C擾
char *CFG6::GetExtRomFile( void )
{
	Ini->GetString( "FILES", "ExtRom", ExtRomFile, ExtRomFile );
	Delimiter( ExtRomFile );
	return ExtRomFile;
}

// gROMt@Cݒ
void CFG6::SetExtRomFile( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	Ini->PutEntry( "FILES", MSINI_ExtRom, "ExtRom", temp );
}

// CRC`FbN擾
bool CFG6::GetCheckCRC( void )
{
	bool st = true;
	Ini->GetTruth( "CONFIG", "CheckCRC", &st, st );
	return st;
}
// CRC`FbNݒ
void CFG6::SetCheckCRC( bool yn )
{
	Ini->PutEntry( "CONFIG", MSINI_CheckCRC, "CheckCRC", "%s", yn ? "Yes" : "No" );
}

// ROMpb`擾
bool CFG6::GetRomPatch( void )
{
	bool st = true;
	Ini->GetTruth( "CONFIG", "RomPatch", &st, st );
	return st;
}
// ROMpb`ݒ
void CFG6::SetRomPatch( bool yn )
{
	Ini->PutEntry( "CONFIG", MSINI_RomPatch, "RomPatch", "%s", yn ? "Yes" : "No" );
}

// L[s[g擾
int CFG6::GetKeyRepeat( void )
{
	int st = DEFAULT_REPEAT;
	Ini->GetInt( "KEY", "KeyRepeat", &st, st );
	return st;
}

// L[s[gݒ
void CFG6::SetKeyRepeat( int data )
{
	Ini->PutEntry( "KEY", MSINI_KeyRepeat, "KeyRepeat", "%d", data );
}

// TvO[g擾
int CFG6::GetSampleRate( void )
{
	int st = DEFAULT_SAMPLE_RATE;
	Ini->GetInt( "SOUND", "SampleRate", &st, st );
	return st;
}

// TvO[gݒ
void CFG6::SetSampleRate( int data )
{
	Ini->PutEntry( "SOUND", MSINI_SampleRate, "SampleRate", "%d", data );
}

// TEhobt@{擾
int CFG6::GetSoundBuffer( void )
{
	int st = SOUND_BUFFER_SIZE;
	Ini->GetInt( "SOUND", "SoundBuffer", &st, st );
	return st;
}

// TEhobt@{ݒ
void CFG6::SetSoundBuffer( int data )
{
	Ini->PutEntry( "SOUND", MSINI_SoundBuffer, "SoundBuffer", "%d", data );
}

// }X^[ʎ擾
int CFG6::GetMasterVol( void )
{
	int st = DEFAULT_MASTERVOL;
	Ini->GetInt( "SOUND", "MasterVolume", &st, st );
	return st;
}

// }X^[ʐݒ
void CFG6::SetMasterVol( int data )
{
	Ini->PutEntry( "SOUND", MSINI_MasterVolume, "MasterVolume", "%d", data );
}


// PSGʎ擾
int CFG6::GetPsgVol( void )
{
	int st = DEFAULT_PSGVOL;
	Ini->GetInt( "SOUND", "PsgVolume", &st, st );
	return st;
}

// PSGʐݒ
void CFG6::SetPsgVol( int data )
{
	Ini->PutEntry( "SOUND", MSINI_PsgVolume, "PsgVolume", "%d", data );
}

// PSG LPFJbgItg擾
int CFG6::GetPsgLPF( void )
{
	int st = DEFAULT_PSGLPF;
	Ini->GetInt( "SOUND", "PsgLPF", &st, st );
	return st;
}

// PSG LPFJbgItgݒ
void CFG6::SetPsgLPF( int data )
{
	Ini->PutEntry( "SOUND", MSINI_PsgLPF,  "PsgLPF",  "%d", data );
}

// WAVEpX擾
char *CFG6::GetWavePath( void )
{
	Ini->GetString( "PATH", "WavePath", WavePath, WavePath );
	Delimiter( WavePath );
	AddDelimiter( WavePath );
	return WavePath;
}

// WAVEpXݒ
void CFG6::SetWavePath( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "PATH", MSINI_WavePath, "WavePath", temp );
}

// ʎ擾
int CFG6::GetVoiceVol( void )
{
	int st = DEFAULT_VOICEVOL;
	Ini->GetInt( "SOUND", "VoiceVolume", &st, st );
	return st;
}

// ʐݒ
void CFG6::SetVoiceVol( int data )
{
	Ini->PutEntry( "SOUND", MSINI_VoiceVolume, "VoiceVolume", "%d", data );
}

// TAPEpX擾
char *CFG6::GetTapePath( void )
{
	Ini->GetString( "PATH", "TapePath", TapePath, TapePath );
	Delimiter( TapePath );
	AddDelimiter( TapePath );
	return TapePath;
}

// TAPEpXݒ
void CFG6::SetTapePath( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "PATH", MSINI_TapePath, "TapePath", temp );
}

// TAPEt@C擾
char *CFG6::GetTapeFile( void )
{
	Ini->GetString( "FILES", "tape", TapeFile, TapeFile );
	Delimiter( TapeFile );
	return TapeFile;
}

// TAPEt@Cݒ
void CFG6::SetTapeFile( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "FILES", MSINI_tape, "tape", temp );
}

// TAPEj^ʎ擾
int CFG6::GetCmtVol( void )
{
	int st = DEFAULT_TAPEVOL;
	Ini->GetInt( "SOUND", "TapeVolume", &st, st );
	return st;
}

// TAPEj^ʐݒ
void CFG6::SetCmtVol( int data )
{
	Ini->PutEntry( "SOUND", MSINI_TapeVolume, "TapeVolume", "%d", data );
}

// Turbo TAPE LtO擾
bool CFG6::GetTurboTAPE( void )
{
	bool st = DEFAULT_TURBO;
	Ini->GetTruth( "CONFIG", "TurboTAPE", &st, st );
	return st;
}

// Turbo TAPE LtOݒ
void CFG6::SetTurboTAPE( bool yn )
{
	Ini->PutEntry( "CONFIG", MSINI_TurboTAPE, "TurboTAPE", "%s", yn ? "Yes" : "No" );
}

// Boost Up LtO擾
bool CFG6::GetBoostUp( void )
{
	bool st = DEFAULT_BOOST;
	Ini->GetTruth( "CONFIG", "BoostUp", &st, st );
	return st;
}

// BoostUp LtOݒ
void CFG6::SetBoostUp( bool yn )
{
	Ini->PutEntry( "CONFIG", MSINI_BoostUp, "BoostUp", "%s", yn ? "Yes" : "No" );
}

// BoostUp ő{(N60[h)擾
int CFG6::GetMaxBoost1( void )
{
	int st = DEFAULT_MAXBOOST60;
	Ini->GetInt( "CONFIG", "MaxBoost60", &st, st );
	return st;
}

// BoostUp ő{(N60[h)ݒ
void CFG6::SetMaxBoost1( int data )
{
	Ini->PutEntry( "CONFIG", MSINI_MaxBoost60, "MaxBoost60", "%d", data );
}

// BoostUp ő{(N60m/N66[h)擾
int CFG6::GetMaxBoost2( void )
{
	int st = DEFAULT_MAXBOOST62;
	Ini->GetInt( "CONFIG", "MaxBoost62", &st, st );
	return st;
}
// BoostUp ő{(N60m/N66[h)ݒ
void CFG6::SetMaxBoost2( int data )
{
	Ini->PutEntry( "CONFIG", MSINI_MaxBoost62, "MaxBoost62", "%d", data );
}

// TAPE(SAVE)t@C擾
char *CFG6::GetSaveFile( void )
{
	Ini->GetString( "FILES", "save", SaveFile, SaveFile );
	Delimiter( SaveFile );
	return SaveFile;
}

// TAPE(SAVE)t@Cݒ
void CFG6::SetSaveFile( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "FILES", MSINI_save, "save", temp );
}

// DISKpX擾
char *CFG6::GetDiskPath( void )
{
	Ini->GetString( "PATH", "DiskPath", DiskPath, DiskPath );
	Delimiter( DiskPath );
	AddDelimiter( DiskPath );
	return DiskPath;
}

// DISKpXݒ
void CFG6::SetDiskPath( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "PATH", MSINI_DiskPath, "DiskPath", temp );
}

// DISKt@C擾
char *CFG6::GetDiskFile( int drv )
{
	switch( drv ){
	case 1:	Ini->GetString( "FILES", "disk1", DiskFile1, DiskFile1 );
			Delimiter( DiskFile1 );
			return DiskFile1;
	case 2: Ini->GetString( "FILES", "disk2", DiskFile2, DiskFile2 );
			Delimiter( DiskFile2 );
			return DiskFile2;
	}
	return NULL;
}

// DISKt@Cݒ
void CFG6::SetDiskFile( int drv, const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	switch( drv ){
	case 1: Ini->PutEntry( "FILES", MSINI_disk1, "disk1", temp ); break;
	case 2: Ini->PutEntry( "FILES", MSINI_disk2, "disk2", temp ); break;
	}
}

// FDDڑ䐔擾
int CFG6::GetFddNum( void )
{
	int st = DEFAULT_FDD;
	Ini->GetInt( "CONFIG", "FDD", &st, st );
	return st;
}

// FDDڑ䐔ݒ
void CFG6::SetFddNum( int data )
{
	Ini->PutEntry( "CONFIG", MSINI_FDD, "FDD", "%d", data );
}

// XN[Vbgi[pX擾
char *CFG6::GetImgPath( void )
{
	Ini->GetString( "PATH", "ImgPath", ImgPath, ImgPath );
	Delimiter( ImgPath );
	AddDelimiter( ImgPath );
	return ImgPath;
}

// XN[Vbgi[pXݒ
void CFG6::SetImgPath( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "PATH", MSINI_ImgPath, "ImgPath", temp );
}

// J[[h擾
int CFG6::GetScrBpp( void )
{
	int st = DEFAULT_COLOR_MODE;
	Ini->GetInt( "DISPLAY", "ScrBpp", &st, st );
	return st;
}

// J[[hݒ
void CFG6::SetScrBpp( int data )
{
	Ini->PutEntry( "DISPLAY", MSINI_ScrBpp, "ScrBpp", "%d", data );
}

// [h4J[[h擾
int CFG6::GetMode4Color( void )
{
	int st = DEFAULT_MODE4_COLOR;
	Ini->GetInt( "DISPLAY", "Mode4Color", &st, st );
	return st;
}

// [h4J[[hݒ
void CFG6::SetMode4Color( int data )
{
	Ini->PutEntry( "DISPLAY", MSINI_Mode4Color, "Mode4Color", "%d", data );
}

// XLC擾
bool CFG6::GetScanLine( void )
{
	bool st = DEFAULT_SCANLINE;
	Ini->GetTruth( "DISPLAY", "ScanLine", &st, st );
	return st;
}

// XLCݒ
void CFG6::SetScanLine( bool yn )
{
	Ini->PutEntry( "DISPLAY", MSINI_ScanLine, "ScanLine", "%s", yn ? "Yes" : "No" );
}

// XLCPx擾
int CFG6::GetScanLineBr( void )
{
	int st = DEFAULT_SCANLINEBR;
	Ini->GetInt( "DISPLAY", "ScanLineBr", &st, st );
	return st;
}

// XLCPxݒ
void CFG6::SetScanLineBr( int data )
{
	Ini->PutEntry( "DISPLAY", MSINI_ScanLineBr, "ScanLineBr", "%d", data );
}

// 4:3\擾
bool CFG6::GetDispNTSC( void )
{
	bool st = DEFAULT_DISPNTSC;
	Ini->GetTruth( "DISPLAY", "DispNTSC", &st, st );
	return st;
}

// 4:3\ݒ
void CFG6::SetDispNTSC( bool yn )
{
	Ini->PutEntry( "DISPLAY", MSINI_DispNTSC, "DispNTSC", "%s", yn ? "Yes" : "No" );
}

// tXN[擾
bool CFG6::GetFullScreen( void )
{
	bool st = false;
	Ini->GetTruth( "DISPLAY", "FullScreen", &st, st );
	return st;
}

// tXN[ݒ
void CFG6::SetFullScreen( bool yn )
{
	Ini->PutEntry( "DISPLAY", MSINI_FullScreen, "FullScreen", "%s", yn ? "Yes" : "No" );
}

// Xe[^Xo[\Ԏ擾
bool CFG6::GetStatDisp( void )
{
	bool st = true;
	Ini->GetTruth( "DISPLAY", "DispStatus", &st, st );
	return st;
}

// Xe[^Xo[\Ԑݒ
void CFG6::SetStatDisp( bool yn )
{
	Ini->PutEntry( "DISPLAY", MSINI_DispStatus, "DispStatus", "%s", yn ? "Yes" : "No" );
}

// t[XLbv擾
int CFG6::GetFrameSkip( void )
{
	int st = DEFAULT_FRAMESKIP;
	Ini->GetInt( "DISPLAY", "FrameSkip", &st, st );
	return st;
}

// t[XLbvݒ
void CFG6::SetFrameSkip( int data )
{
	Ini->PutEntry( "DISPLAY", MSINI_FrameSkip, "FrameSkip", "%d", data );
}

// RLEtO擾
bool CFG6::GetAviRle()
{
	bool st = true;
	Ini->GetTruth( "MOVIE", "AviRle", &st, st );
	return st;
}

// RLEtOݒ
void CFG6::SetAviRle( bool yn )
{
	Ini->PutEntry( "MOVIE", MSINI_AviRle, "AviRle", "%s", yn ? "Yes" : "No" );
}

// v^t@C擾
char *CFG6::GetPrinterFile( void )
{
	Ini->GetString( "FILES", "printer", PrinterFile, PrinterFile );
	Delimiter( PrinterFile );
	return PrinterFile;
}

// v^t@Cݒ
void CFG6::SetPrinterFile( const char *str )
{
	char temp[PATH_MAX];
	strncpy( temp, str, PATH_MAX );
	UnDelimiter( temp );
	DelDelimiter( temp );
	Ini->PutEntry( "FILES", MSINI_printer, "printer", temp );
}

// ImF擾
bool CFG6::GetCkQuit( void )
{
	bool st = false;
	Ini->GetTruth( "CHECK", "CkQuit", &st, st );
	return st;
}

// ImFݒ
void CFG6::SetCkQuit( bool yn )
{
	Ini->PutEntry( "CHECK", MSINI_CkQuit, "CkQuit", "%s", yn ? "Yes" : "No" );
}

// IINIۑ擾
bool CFG6::GetSaveQuit( void )
{
	bool st = false;
	Ini->GetTruth( "CHECK", "SaveQuit", &st, st );
	return st;
}

// IINIۑݒ
void CFG6::SetSaveQuit( bool yn )
{
	Ini->PutEntry( "CHECK", MSINI_SaveQuit, "SaveQuit", "%s", yn ? "Yes" : "No" );
}

// m̃J[gbWgtO擾
bool CFG6::GetUseSoldier()
{
	bool st = true;
	Ini->GetTruth( "OPTION", "UseSoldier", &st, st );
	return st;
}

// m̃J[gbWgtOݒ
void CFG6::SetUseSoldier( bool yn )
{
	Ini->PutEntry( "OPTION", MSINI_UseSoldier, "UseSoldier", "%s", yn ? "Yes" : "No" );
}

// J[f[^擾
COLOR24 *CFG6::GetColor( int num, COLOR24 *col )
{
	if( num < (int)(sizeof(STDColor)/sizeof(STDColor[0])) ){
		char strin[16], str[PATH_MAX];
		sprintf( strin, "COL%03d", num );
		sprintf( str,   "%02X%02X%02X", STDColor[num].r, STDColor[num].g, STDColor[num].b );
		Ini->GetString( "COLOR", strin, str, str );
		int st = strtol( str, NULL, 16 );
		col->r = (st>>16)&0xff;
		col->g = (st>> 8)&0xff;
		col->b = (st    )&0xff;
	}else{
		col->r = 0;
		col->g = 0;
		col->b = 0;
	}
	return col;
}

// J[f[^ݒ
void CFG6::SetColor( int num, const COLOR24 *col )
{
	char stren[16];
	sprintf( stren, "COL%03d", num );
	if( col )
		Ini->PutEntry( "COLOR", OSD_ColorName( num-16 ), stren, "%02X%02X%02X", col->r, col->g, col->b );
}

// L[`擾
P6KEYsym CFG6::GetVKey( PCKEYsym pcs )
{
	char str[PATH_MAX] = "";
	
	// L[R[h疼̎擾
	const char *k1 = GetPCKeyName( pcs );
	
	if( k1 ){
		Ini->GetString( "KEY", k1, str, str );
		return GetP6KeyCode( str );
	}
	return KP6_UNKNOWN;
}

// L[`ݒ
void CFG6::SetVKey( PCKEYsym pcs, P6KEYsym p6s )
{
	const char *k1 = GetPCKeyName( pcs );
	const char *k2 = GetP6KeyName( p6s );
	if( k1 && k2 ){
		Ini->PutEntry( "KEY", OSD_KeyName( pcs ), k1, k2 );
	}
}

// L[`z擾
int CFG6::GetVKeyDef( VKeyConv **kdef )
{
	VKeyConv *key = NULL;
	
	try{
		// L[`e[u쐬
		key = new VKeyConv[COUNTOF(KeyIni)];
		for( int i=0; i<COUNTOF(KeyIni); i++ ){
			key[i].PCKey = KeyIni[i].PCKey;
			key[i].P6Key = GetVKey( key[i].PCKey );
		}
	}
	catch( std::bad_alloc ){	// new Ɏsꍇ
		Error::SetError( Error::MemAllocFailed );
		*kdef = NULL;
		return 0;
	}
	catch( Error::Errno i ){	// O
		Error::SetError( i );
		if( key ) delete [] key;
		*kdef = NULL;
		return 0;
	}
	
	*kdef = key;
	return COUNTOF(KeyIni);
}

#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
// j^EBhE\Ԏ擾
bool CFG6::GetMonDisp( void )
{
	return MonDisp;
}

// j^EBhE\Ԑݒ
void CFG6::SetMonDisp( bool yn )
{
	MonDisp = yn;
}


#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //


////////////////////////////////////////////////////////////////
// INIIuWFNglݒ
//
// :	ini		INIIuWFNgւ̃|C^
//			over	true:㏑ false:m[h݂ĂpX
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void CFG6::InitIni( cIni *ini, bool over )
{
	if( !ini ) return;
	
	char str[PATH_MAX];
	
	// [CONFIG]
	// @
	if( over || !ini->GetString( "CONFIG", "Model", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_Model,		"Model",		"%02d",	DEFAULT_MODEL );
	
	// FDD
	if( over || !ini->GetString( "CONFIG", "FDD", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_FDD,			"FDD",			"%d",	DEFAULT_FDD );
	
	// gRAMgp
	if( over || !ini->GetString( "CONFIG", "ExtRam", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_ExtRam,		"ExtRam",		"%s",	DEFAULT_EXTRAM ? "Yes" : "No" );
	
	// Turbo TAPE
	if( over || !ini->GetString( "CONFIG", "TurboTAPE", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_TurboTAPE,	"TurboTAPE",	"%s",	DEFAULT_TURBO ? "Yes" : "No" );
	
	// Boost Up
	if( over || !ini->GetString( "CONFIG", "BoostUp", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_BoostUp,		"BoostUp",		"%s",	DEFAULT_BOOST ? "Yes" : "No" );
	
	// BoostUp ő{(N60[h)
	if( over || !ini->GetString( "CONFIG", "MaxBoost60", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_MaxBoost60,	"MaxBoost60",	"%d",	DEFAULT_MAXBOOST60 );
	
	// BoostUp ő{(N60m/N66[h)
	if( over || !ini->GetString( "CONFIG", "MaxBoost62", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_MaxBoost62,	"MaxBoost62",	"%d",	DEFAULT_MAXBOOST62 );
	
	// I[o[NbN
	if( over || !ini->GetString( "CONFIG", "OverClock", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_OverClock,	"OverClock",	"%d",	DEFAULT_OVERCLOCK );
	
	// CRC`FbN
	if( over || !ini->GetString( "CONFIG", "CheckCRC", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_CheckCRC,	"CheckCRC",		"%s",	DEFAULT_CHECKCRC ? "Yes" : "No" );
	
	// ROMpb`
	if( over || !ini->GetString( "CONFIG", "RomPatch", str, str ) )
		ini->PutEntry( "CONFIG", MSINI_RomPatch,	"RomPatch",		"%s",	DEFAULT_ROMPATCH ? "Yes" : "No" );
	
	// [DISPLAY]
	// J[[h
	if( over || !ini->GetString( "DISPLAY", "ScrBpp", str, str ) )
		ini->PutEntry( "DISPLAY", MSINI_ScrBpp,		"ScrBpp",		"%d",	DEFAULT_COLOR_MODE );
	
	// MODE4J[
	if( over || !ini->GetString( "DISPLAY", "Mode4Color", str, str ) )
		ini->PutEntry( "DISPLAY", MSINI_Mode4Color,	"Mode4Color",	"%d",	DEFAULT_MODE4_COLOR );
	
	// XLC
	if( over || !ini->GetString( "DISPLAY", "ScanLine", str, str ) )
		ini->PutEntry( "DISPLAY", MSINI_ScanLine,	"ScanLine",		"%s",	DEFAULT_SCANLINE ? "Yes" : "No" );
	
	// XLCPx
	if( over || !ini->GetString( "DISPLAY", "ScanLineBr", str, str ) )
		ini->PutEntry( "DISPLAY", MSINI_ScanLineBr,	"ScanLineBr",	"%d",	DEFAULT_SCANLINEBR );
	
	// 4:3\
	if( over || !ini->GetString( "DISPLAY", "DispNTSC", str, str ) )
		ini->PutEntry( "DISPLAY", MSINI_DispNTSC,	"DispNTSC",		"%s",	DEFAULT_DISPNTSC ? "Yes" : "No" );
	
	// tXN[
	if( over || !ini->GetString( "DISPLAY", "FullScreen", str, str ) )
		ini->PutEntry( "DISPLAY", MSINI_FullScreen,	"FullScreen",	"%s",	DEFAULT_FULLSCREEN ? "Yes" : "No" );
	
	// Xe[^Xo[\
	if( over || !ini->GetString( "DISPLAY", "DispStatus", str, str ) )
		ini->PutEntry( "DISPLAY", MSINI_DispStatus,	"DispStatus",	"%s",	DEFAULT_DISPSTATUS ? "Yes" : "No" );
	
	// t[XLbv
	if( over || !ini->GetString( "DISPLAY", "FrameSkip", str, str ) )
		ini->PutEntry( "DISPLAY", MSINI_FrameSkip,	"FrameSkip",	"%d",	DEFAULT_FRAMESKIP );
	
	
	// [SOUND]
	// TvO[g
	if( over || !ini->GetString( "SOUND", "SampleRate", str, str ) )
		ini->PutEntry( "SOUND", MSINI_SampleRate,	"SampleRate",	"%d",	DEFAULT_SAMPLE_RATE );
	
	// TEhobt@{
	if( over || !ini->GetString( "SOUND", "SoundBuffer", str, str ) )
		ini->PutEntry( "SOUND", MSINI_SoundBuffer,	"SoundBuffer",	"%d",	SOUND_BUFFER_SIZE );
	
	// }X^[
	if( over || !ini->GetString( "SOUND", "MasterVolume", str, str ) )
		ini->PutEntry( "SOUND", MSINI_MasterVolume,	"MasterVolume",	"%d",	DEFAULT_MASTERVOL );
	
	// PSG
	if( over || !ini->GetString( "SOUND", "PsgVolume", str, str ) )
		ini->PutEntry( "SOUND", MSINI_PsgVolume,	"PsgVolume",	"%d",	DEFAULT_PSGVOL );
	
	// PSG LPFJbgItg
	if( over || !ini->GetString( "SOUND", "PsgLPF", str, str ) )
		ini->PutEntry( "SOUND", MSINI_PsgLPF,		"PsgLPF",		"%d",	DEFAULT_PSGLPF );
	
	// 
	if( over || !ini->GetString( "SOUND", "VoiceVolume", str, str ) )
		ini->PutEntry( "SOUND", MSINI_VoiceVolume,	"VoiceVolume",	"%d",	DEFAULT_VOICEVOL );
	
	// TAPEj^
	if( over || !ini->GetString( "SOUND", "TapeVolume", str, str ) )
		ini->PutEntry( "SOUND", MSINI_TapeVolume,	"TapeVolume",	"%d",	DEFAULT_TAPEVOL );
	
	
	// [MOVIE]
	// RLEtO
	if( over || !ini->GetString( "MOVIE", "AviRle", str, str ) )
		ini->PutEntry( "MOVIE", MSINI_AviRle,		"AviRle",		"%s",	DEFAULT_AVIRLE ? "Yes" : "No" );
	
	
	//[FILES]
	// gROMt@C(NɎ}Eg)
	if( over || !ini->GetString( "FILES", "ExtRom", str, str ) )
		ini->PutEntry( "FILES",	MSINI_ExtRom,	"ExtRom",	"" );
	
	// TAPEt@C(NɎ}Eg)
	if( over || !ini->GetString( "FILES", "tape", str, str ) )
		ini->PutEntry( "FILES", MSINI_tape,		"tape",		"" );
	
	// TAPE(SAVE)t@C(SAVEɎ}Eg)
	if( over || !ini->GetString( "FILES", "save", str, str ) ){
		sprintf( str, "%s%s/%s", OSD_GetModulePath(), TAPE_DIR, SAVE_FILE );
		UnDelimiter( str );
		ini->PutEntry( "FILES",	MSINI_save,		"save",		str );
	}
	
	// DISK1t@C(NɎ}Eg)
	if( over || !ini->GetString( "FILES", "disk1", str, str ) )
		ini->PutEntry( "FILES",	MSINI_disk1,	"disk1",	"" );
	
	// DISK2t@C(NɎ}Eg)
	if( over || !ini->GetString( "FILES", "disk2", str, str ) )
		ini->PutEntry( "FILES",	MSINI_disk2,	"disk2",	"" );
	
	// v^t@C
	if( over || !ini->GetString( "FILES", "printer", str, str ) ){
		sprintf( str, "%s" PRINTER_FILE, OSD_GetModulePath() );
		UnDelimiter( str );
		ini->PutEntry( "FILES",	MSINI_printer,	"printer",	str );
	}
	
	// [PATH]
	// ROMpX
	if( over || !ini->GetString( "PATH", "RomPath", str, str ) ){
		sprintf( str, "%s" ROM_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		ini->PutEntry( "PATH",	MSINI_RomPath,	"RomPath",	str );
	}
	
	// TAPEpX
	if( over || !ini->GetString( "PATH", "TapePath", str, str ) ){
		sprintf( str, "%s" TAPE_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		ini->PutEntry( "PATH",	MSINI_TapePath,	"TapePath",	str );
	}
	
	// DISKpX
	if( over || !ini->GetString( "PATH", "DiskPath", str, str ) ){
		sprintf( str, "%s" DISK_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		ini->PutEntry( "PATH",	MSINI_DiskPath,	"DiskPath",	str );
	}
	
	// gROMpX
	if( over || !ini->GetString( "PATH", "ExtRomPath", str, str ) ){
		sprintf( str, "%s" EXTROM_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		ini->PutEntry( "PATH",	MSINI_ExtRomPath,	"ExtRomPath",	str );
	}
	
	// WAVEpX
	if( over || !ini->GetString( "PATH", "WavePath", str, str ) ){
		sprintf( str, "%s" WAVE_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		ini->PutEntry( "PATH",	MSINI_WavePath,	"WavePath",	str );
	}
	
	// IMGpX
	if( over || !ini->GetString( "PATH", "ImgPath", str, str ) ){
		sprintf( str, "%s" IMAGE_DIR, OSD_GetModulePath() );
		UnDelimiter( str );
		ini->PutEntry( "PATH",	MSINI_ImgPath,	"ImgPath",	str );
	}
	
	// [CHECK]
	// ImF
	if( over || !ini->GetString( "CHECK", "CkQuit", str, str ) )
		ini->PutEntry( "CHECK", MSINI_CkQuit,		"CkQuit",		"%s",	DEFAULT_CKQUIT ? "Yes" : "No" );
	
	// IINIۑ
	if( over || !ini->GetString( "CHECK", "SaveQuit", str, str ) )
		ini->PutEntry( "CHECK", MSINI_SaveQuit,		"SaveQuit",		"%s",	DEFAULT_SAVEQUIT ? "Yes" : "No" );
	
	
	// [OPTION]
	// m̃J[gbWgtO
	if( over || !ini->GetString( "OPTION", "UseSoldier", str, str ) )
		ini->PutEntry( "OPTION", MSINI_UseSoldier,	"UseSoldier",	"%s",	DEFAULT_SOLDIER ? "Yes" : "No" );
	
	
	// [COLOR]
	// pbg
	for( int i=16; i<COUNTOF(STDColor); i++ ){
		char stren[16];
		sprintf( stren, "COL%03d", i );
		if( over || !ini->GetString( "COLOR", stren, str, str ) )
			ini->PutEntry( "COLOR", OSD_ColorName( i-16 ), stren, "%02X%02X%02X", STDColor[i].r, STDColor[i].g, STDColor[i].b );
	}
	
	// [KEY]
	// L[s[g
	if( over || !ini->GetString( "KEY", "KeyRepeat", str, str ) )
		ini->PutEntry( "KEY", MSINI_KeyRepeat,		"KeyRepeat",	"%d",	DEFAULT_REPEAT );
	
	// L[`
	int i = 0;
	while( KeyIni[i].PCKey != KVC_LAST ){
		// L[R[h疼̎擾
		const char *k1 = GetPCKeyName( KeyIni[i].PCKey );
		const char *k2 = GetP6KeyName( KeyIni[i].P6Key );
		if( k1 && k2 ){
			if( over || !ini->GetString( "KEY", k1, str, str ) )
				ini->PutEntry( "KEY", OSD_KeyName( KeyIni[i].PCKey ), k1, k2 );
		}
		i++;
	}
	
}


////////////////////////////////////////////////////////////////
// zL[R[h疼̎擾
//
// :	sym		zL[R[h
// Ԓl:	char *	̕ւ̃|C^(ȂNULL)
////////////////////////////////////////////////////////////////
const char *CFG6::GetPCKeyName( PCKEYsym sym )
{
	const char *str = NULL;
	for( int i=0; i<(int)(sizeof(PCKeyNameDef)/sizeof(PCKeyName)); i++ ){
		if( PCKeyNameDef[i].PCKey == sym ){
			str = PCKeyNameDef[i].Name;
			break;
		}
	}
	return str;
}


////////////////////////////////////////////////////////////////
// P6L[R[h疼̎擾
//
// :	sym		P6L[R[h
// Ԓl:	char *	̕ւ̃|C^(ȂNULL)
////////////////////////////////////////////////////////////////
const char *CFG6::GetP6KeyName( P6KEYsym sym )
{
	const char *str = NULL;
	for( int i=0; i<(int)(sizeof(P6KeyNameDef)/sizeof(P6KeyName)); i++ ){
		if( P6KeyNameDef[i].P6Key == sym ){
			str = P6KeyNameDef[i].Name;
			break;
		}
	}
	return str;
}


////////////////////////////////////////////////////////////////
// L[̂牼zL[R[h擾
//
// :	str			̕ւ̃|C^
// Ԓl:	PCKEYsym	zL[R[h
////////////////////////////////////////////////////////////////
PCKEYsym CFG6::GetPCKeyCode( char *str )
{
	PCKEYsym sym = KVC_UNKNOWN;
	for( int i=0; i<(int)(sizeof(PCKeyNameDef)/sizeof(PCKeyName)); i++ ){
		if( !stricmp( PCKeyNameDef[i].Name, str ) ){
			sym = PCKeyNameDef[i].PCKey;
			break;
		}
	}
	return sym;
}


////////////////////////////////////////////////////////////////
// L[̂P6L[R[h擾
//
// :	str			̕ւ̃|C^
// Ԓl:	P6KEYsym	P6L[R[h
////////////////////////////////////////////////////////////////
P6KEYsym CFG6::GetP6KeyCode( char *str )
{
	P6KEYsym sym = KP6_UNKNOWN;
	for( int i=0; i<(int)(sizeof(P6KeyNameDef)/sizeof(P6KeyName)); i++ ){
		if( !stricmp( P6KeyNameDef[i].Name, str ) ){
			sym = P6KeyNameDef[i].P6Key;
			break;
		}
	}
	return sym;
}


////////////////////////////////////////////////////////////////
// ǂłSAVE
//
// :	Ini		INIIuWFNg|C^
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool CFG6::DokoSave( cIni *Ini )
{
	if( !Ini ) return false;
	
	// 
	Ini->PutEntry( "GLOBAL", NULL, "Version",	VERSION );
	Ini->PutEntry( "GLOBAL", NULL, "P6Model",	"%02d",	GetModel() );
	
	return true;
}


////////////////////////////////////////////////////////////////
// ǂłLOAD
//
// :	Ini		INIIuWFNg|C^
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool CFG6::DokoLoad( cIni *Ini )
{
	int st;
	char strva[256];
	
	if( !Ini ) return false;
	
	// 
	Ini->GetString( "GLOBAL", "Version", strva, "" );
	if( strcmp( strva, VERSION ) ){
		Error::SetError( Error::DokoDiffVersion );
		return false;
	}
	
	Ini->GetInt( "GLOBAL", "P6Model",	&st, 0 );	SetModel( st );
	
	return true;
}
