#include "config.h"
#include "log.h"
#include "cpus.h"
#include "intr.h"
#include "io.h"
#include "keyboard.h"
#include "memory.h"
#include "schedule.h"
#include "tape.h"


// CxgID
#define	EID_INTCHK	(1)
#define	EID_VECTOR	(2)
#define	EID_DATA	(3)

// EFCg(us)
#define	WAIT_INTCHK	(300)
// EFCg(Z80Xe[g)
#define	WAIT_DATA	(15)

// 8049Xe[^X
#define	D8049_IDLE		(0)				/* ĂȂ */
#define	D8049_IMASK		(0xff00)		/* O݃Xe[^X}XN */
#define	D8049_INTR		(0x0100)		/* Oݗv */

#define	D8049_CMTO		(0x0200)		/* CMT 1o f[^҂ */

#define	D8049_KEY1		(0x0001)		/* L[11 */
#define	D8049_KEY12		(0x0002)		/* L[12 */
#define	D8049_KEY2		(0x0003)		/* L[2 */
#define	D8049_KEY3		(0x0004)		/* L[3 */
#define	D8049_CMTR		(0x0005)		/* CMT READ */
#define	D8049_CMTE		(0x0006)		/* CMT ERROR */
#define	D8049_SIO		(0x0007)		/* RS232CM */
#define	D8049_JOY		(0x0008)		/* Q[pL[ */

#define	D8049_NODATA	(0x0009)		/* f[^o͂Ȃ */

// ݗvtO
#define	IR_KEY1			(0x0001)
#define	IR_KEY12		(0x0002)
#define	IR_CMTR			(0x0004)
#define	IR_CMTE			(0x0008)
#define	IR_KEY2			(0x0010)
#define	IR_KEY3			(0x0020)
#define	IR_SIO			(0x0040)
#define	IR_JOY			(0x0080)

// ݃xN^
#define	IVEC_NOINTR		(0xff)
#define	IVEC_SIO		(0)
#define	IVEC_KEY3		(2)
#define	IVEC_TIMER		(6)
#define	IVEC_CMT_R		(8)
#define	IVEC_KEY1		(14)
#define	IVEC_KEY12		(16)
#define	IVEC_CMT_E		(18)
#define	IVEC_KEY2		(20)
#define	IVEC_JOY		(22)


////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
SUB6::SUB6( VM6 *vm, const P6ID& id ) : P6DEVICE(vm,id),
	CmtStatus(CMTCLOSE), Status8049(D8049_IDLE),
	IntrFlag(0), KeyCode(0), JoyCode(0), CmtData(0), SioData(0) {}

SUB60::SUB60( VM6 *vm, const P6ID& id ) : SUB6(vm,id){}
SUB62::SUB62( VM6 *vm, const P6ID& id ) : SUB6(vm,id){}

////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
SUB6::~SUB6( void ){}
SUB60::~SUB60( void ){}
SUB62::~SUB62( void ){}


////////////////////////////////////////////////////////////////
// CxgR[obN֐
//
// :	id		CxgID
//			clock	NbN
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void SUB6::EventCallback( int id, int clock )
{
	switch( id ){
	case EID_INTCHK:	// ݃`FbN
		if( (Status8049 == D8049_IDLE) && IntrFlag ){
			// ݗv𒲂ׂ
			if     ( IntrFlag & IR_JOY )  { Status8049 = D8049_JOY;   }
			else if( IntrFlag & IR_KEY1 ) { Status8049 = D8049_KEY1;  }
			else if( IntrFlag & IR_KEY12 ){ Status8049 = D8049_KEY12; }
			else if( IntrFlag & IR_CMTR ) { Status8049 = D8049_CMTR;  }
			else if( IntrFlag & IR_CMTE ) { Status8049 = D8049_CMTE;  }
			else if( IntrFlag & IR_KEY2 ) { Status8049 = D8049_KEY2;  }
			else if( IntrFlag & IR_KEY3 ) { Status8049 = D8049_KEY3;  }
			else if( IntrFlag & IR_SIO )  { Status8049 = D8049_SIO;   }
			
			if( Status8049 != D8049_IDLE )
				vm->evsc->Add( this, EID_VECTOR, WAIT_DATA, EV_LOOP|EV_STATE );
		}
		break;
		
	case EID_VECTOR:	// ݃xN^o
		// T0=IBF=H ܂8255̃obt@󂢂ĂȂ΃gC
		if( GetT0() ) break;
		vm->evsc->Del( this, EID_VECTOR );
		
		OutVector();	// ݃xN^o
		
		// ăf[^o
		vm->evsc->Add( this, EID_DATA, WAIT_DATA, EV_LOOP|EV_STATE );
		break;
		
	case EID_DATA:		// ݃f[^o
		// T0=IBF=H ܂8255̃obt@󂢂ĂȂ΃gC
		if( GetT0() ) break;
		vm->evsc->Del( this, EID_DATA );
		
		// CPUɑ΂銄ݗvLZ(݋֎~΍)
		vm->intr->CancelIntr( IREQ_8049 );
		
		OutData();		// ݃f[^o
		break;
	}
}


////////////////////////////////////////////////////////////////
// Zbg
////////////////////////////////////////////////////////////////
void SUB6::Reset( void )
{
	IntrFlag   = 0;
	Status8049 = D8049_IDLE;
	CmtStatus  = CMTCLOSE;
	
	KeyCode    = 0;
	JoyCode    = 0;
	CmtData    = 0;
	SioData    = 0;
	
	vm->evsc->Add( this, EID_INTCHK, WAIT_INTCHK, EV_LOOP|EV_US );
}


////////////////////////////////////////////////////////////////
// I/O|[gANZX
////////////////////////////////////////////////////////////////
inline BYTE SUB6::ReadIO( int addr )
{
	return vm->ios->In( addr );
}

inline void SUB6::WriteIO( int addr, BYTE data )
{
	vm->ios->Out( addr, data );
}


////////////////////////////////////////////////////////////////
// OANZX
////////////////////////////////////////////////////////////////
inline void SUB6::WriteExt( BYTE data )
{
	vm->ios->Out( IO8049_BUS, data );
}

inline BYTE SUB6::ReadExt( void )
{
	return vm->ios->In( IO8049_BUS );
}


////////////////////////////////////////////////////////////////
// eXg0Xe[^X擾
////////////////////////////////////////////////////////////////
inline bool SUB6::GetT0( void )
{
	return vm->ios->In( IO8049_T0 ) ? true : false;
}


////////////////////////////////////////////////////////////////
// eXg1Xe[^X擾
////////////////////////////////////////////////////////////////
inline bool SUB6::GetT1( void )
{
	return 0;
}


////////////////////////////////////////////////////////////////
// O݃Xe[^X擾
////////////////////////////////////////////////////////////////
inline bool SUB6::GetINT( void )
{
	return vm->ios->In( IO8049_INT ) ? true : false;
}


////////////////////////////////////////////////////////////////
// Oݗv
////////////////////////////////////////////////////////////////
void SUB6::ExtIntr( void )
{
//	static bool CmtPut = false;
	
	// O(8255PortA)
	BYTE comm = ReadExt();
	
	// Oݏ?
	if( Status8049 & D8049_IMASK ){
		// CMT 1o
		if( ( Status8049 & D8049_IMASK ) == D8049_CMTO ){
			PRINTD( SUB_LOG, "[8049][ExtIntr] CmtPut %02X\n", comm );
			
			vm->cmts->CmtWrite( comm );
			Status8049 &= ~D8049_IMASK;
		}
		
	}else{
		// Oݏ
		ExtIntrExec( comm );
	}
}


////////////////////////////////////////////////////////////////
// Oݏ
////////////////////////////////////////////////////////////////
void SUB60::ExtIntrExec( BYTE comm )
{
	PRINTD( SUB_LOG, "[8049][ExtIntrExec] Command %02X\n", comm );
	
	// R}h
	switch( comm ){
	case 0x0c:	// ------ RS-232C ----------
	case 0x2c:
		break;
		
	case 0x06:	// ------ STICK,STRIG ----------
		// Q[pL[ݗv
		ReqJoyIntr();
		break;
		
	case 0x19:	// ------ CMT(LOAD) OPEN ----------
		CmtStatus = LOADOPEN;
		break;
		
	case 0x1a:	// ------ CMT(LOAD) CLOSE ----------
		CmtStatus = CMTCLOSE;
		break;
		
	case 0x1d:	// ------ CMT(LOAD) 600{[ ----------
//		vm->cmtl->SetBaud( 600 );
		break;
		
	case 0x1e:	// ------ CMT(LOAD) 1200{[ ----------
//		vm->cmtl->SetBaud( 1200 );
		break;
		
	case 0x38:	// ------ CMT 1o ----------
		Status8049 |= D8049_CMTO;
		break;
		
	case 0x39:	// ------ CMT(SAVE) OPEN ----------
		vm->cmts->Mount();
		CmtStatus = SAVEOPEN;
		break;
		
	case 0x3a:	// ------ CMT(SAVE) CLOSE ----------
		CmtStatus = CMTCLOSE;
		vm->cmts->Unmount();
		break;
		
	case 0x3d:	// ------ CMT(SAVE) 600{[ ----------
		vm->cmts->SetBaud( 600 );
		break;
		
	case 0x3e:	// ------ CMT(SAVE) 1200{[ ----------
		vm->cmts->SetBaud( 1200 );
		break;
	}
}

void SUB62::ExtIntrExec( BYTE comm )
{
	PRINTD( SUB_LOG, "[8049][ExtIntrExec] Command %02X\n", comm );
	
	// R}h
	switch( comm ){
	case 0x0c:	// ------ RS-232C ----------
	case 0x2c:
		break;
		
	case 0x04:	// ------ p<->Ȑ؊ ----------
		vm->key->ChangeKana();
		break;
		
	case 0x05:	// ------ <->Ji؊ ----------
		vm->key->ChangeKKana();
		break;
		
	case 0x06:	// ------ STICK,STRIG ----------
		// Q[pL[ݗv
		ReqJoyIntr();
		break;
		
	case 0x19:	// ------ CMT(LOAD) OPEN ----------
		CmtStatus = LOADOPEN;
		break;
		
	case 0x1a:	// ------ CMT(LOAD) CLOSE ----------
		CmtStatus = CMTCLOSE;
		break;
		
	case 0x1d:	// ------ CMT(LOAD) 600{[ ----------
//		vm->cmtl->SetBaud( 600 );
		break;
		
	case 0x1e:	// ------ CMT(LOAD) 1200{[ ----------
//		vm->cmtl->SetBaud( 1200 );
		break;
		
	case 0x38:	// ------ CMT 1o ----------
		Status8049 |= D8049_CMTO;
		break;
		
	case 0x39:	// ------ CMT(SAVE) OPEN ----------
		vm->cmts->Mount();
		CmtStatus = SAVEOPEN;
		break;
		
	case 0x3a:	// ------ CMT(SAVE) CLOSE ----------
		CmtStatus = CMTCLOSE;
		vm->cmts->Unmount();
		break;
		
	case 0x3d:	// ------ CMT(SAVE) 600{[ ----------
		vm->cmts->SetBaud( 600 );
		break;
		
	case 0x3e:	// ------ CMT(SAVE) 1200{[ ----------
		vm->cmts->SetBaud( 1200 );
		break;
	}
}


////////////////////////////////////////////////////////////////
// L[ݗv
////////////////////////////////////////////////////////////////
void SUB6::ReqKeyIntr( int flag, BYTE data )
{
	// flag
	// bit 0 : ON_STOP
	// bit 1 : ON_GRAPH
	// bit 2 : ON_FUNC
	
	// ÕL[݂疳
	if( IntrFlag & ( IR_KEY1 | IR_KEY12 | IR_KEY2 | IR_KEY3 ) ) return;
	
	if( flag&1 ){	// STOP ?
		// CMT OPEN ?
		if( CmtStatus ){
			PRINTD( SUB_LOG, "[8049][ReqKeyIntr] KEY1 03\n" );
			IntrFlag |= IR_KEY1;
			KeyCode   = 0x03;
		}else{
			PRINTD( SUB_LOG, "[8049][ReqKeyIntr] KEY2(STOP) FA\n" );
			IntrFlag |= IR_KEY2;
			KeyCode   = 0xfa;
		}
	}else{			// STOPȊO
		// GRAPH or FUNCTION KEY ?
		if( flag&6 ){
			PRINTD( SUB_LOG, "[8049][ReqKeyIntr] KEY2 %02X\n", data );
			IntrFlag |= IR_KEY2;
			KeyCode   = data;
		}else{
			PRINTD( SUB_LOG, "[8049][ReqKeyIntr] KEY3 %02X\n", data );
			IntrFlag |= IR_KEY3;
			KeyCode   = data;
		}
	}
}


////////////////////////////////////////////////////////////////
// CMT READݗv
////////////////////////////////////////////////////////////////
void SUB6::ReqCmtIntr( BYTE data )
{
	PRINTD( SUB_LOG, "[8049][ReqCmtIntr]\n" );
	
	// LOADOPENȂCMTݔ
	if( CmtStatus == LOADOPEN ){
		// OCMT READ݂疳
		if( IntrFlag & IR_CMTR ) return;
		
		IntrFlag |= IR_CMTR;
		CmtData   = data;
	}
}


////////////////////////////////////////////////////////////////
// Q[pL[ݗv
////////////////////////////////////////////////////////////////
void SUB6::ReqJoyIntr( void )
{
	PRINTD( SUB_LOG, "[8049][ReqJoyIntr]\n" );
	
	// ÕQ[pL[݂疳
	if( IntrFlag & IR_JOY ) return;
	
	IntrFlag |= IR_JOY;
	JoyCode   = vm->key->GetKeyJoy();
}


////////////////////////////////////////////////////////////////
// ݃xN^o
////////////////////////////////////////////////////////////////
void SUB6::OutVector( void )
{
	BYTE IntrVector = IVEC_NOINTR;
	
	switch( Status8049 ){
	case D8049_JOY:		// Q[pL[ ݃xN^o
		PRINTD( SUB_LOG, "[OutVector JOY]\n" );
		IntrVector = IVEC_JOY;
		break;
		
	case D8049_KEY1:	// L[1 1 ݃xN^o
		PRINTD( SUB_LOG, "[OutVector KEY1]\n" );
		IntrVector = IVEC_KEY1;
		IntrFlag  &= ~IR_KEY1;
		Status8049 = D8049_NODATA;
		break;
		
	case D8049_KEY12:	// L[1 2 ݃xN^o
		PRINTD( SUB_LOG, "[OutVector KEY12]\n" );
		IntrVector = IVEC_KEY12;
		IntrFlag  &= ~IR_KEY12;
		Status8049 = D8049_NODATA;
		break;
		
	case D8049_CMTR:	// CMT READ ݃xN^o
		PRINTD( SUB_LOG, "[OutVector CMTR]\n" );
		IntrVector = IVEC_CMT_R;
		break;
		
	case D8049_CMTE:	// CMT ERROR ݃xN^o
		PRINTD( SUB_LOG, "[OutVector CMTE]\n" );
		IntrVector = IVEC_CMT_E;
		IntrFlag  &= ~IR_CMTE;
		Status8049 = D8049_NODATA;
		break;
		
	case D8049_KEY2:	// L[2 ݃xN^o
		PRINTD( SUB_LOG, "[OutVector KEY2]\n" );
		IntrVector = IVEC_KEY2;
		break;
		
	case D8049_KEY3:	// L[3 ݃xN^o
		PRINTD( SUB_LOG, "[OutVector KEY3]\n" );
		IntrVector = IVEC_KEY3;
		break;
		
	case D8049_SIO:		// RS232CM ݃xN^o
		PRINTD( SUB_LOG, "[OutVector SIO]\n" );
		IntrVector = IVEC_SIO;
		break;
		
	default:			// ǂłȂ΃xN^o͂Ȃ
		return;
	}
	
	WriteExt( IntrVector );
	vm->intr->ReqIntr( IREQ_8049 );
}


////////////////////////////////////////////////////////////////
// ݃f[^o
////////////////////////////////////////////////////////////////
void SUB6::OutData( void )
{
	switch( Status8049 ){
	case D8049_JOY:		// Q[pL[ f[^o
		PRINTD( SUB_LOG, "[OutData JOY] Data: %02X\n", JoyCode );
		WriteExt( JoyCode );
		IntrFlag &= ~IR_JOY;
		break;
		
	case D8049_KEY1:	// L[1 1 ݃xN^o
		IntrFlag &= ~IR_KEY1;
		break;
		
	case D8049_KEY12:	// L[1 2 ݃xN^o
		IntrFlag &= ~IR_KEY12;
		break;
		
	case D8049_CMTR:	// CMT READ f[^o
		PRINTD( SUB_LOG, "[OutData CMTR] Data: %02X\n", CmtData );
		WriteExt( CmtData );
		IntrFlag &= ~IR_CMTR;
		break;
		
	case D8049_CMTE:	// CMT ERROR ݃xN^o
		IntrFlag &= ~IR_CMTE;
		break;
		
	case D8049_KEY2:	// L[2 f[^o
		PRINTD( SUB_LOG, "[OutData KEY2] Data: %02X\n", KeyCode );
		WriteExt( KeyCode );
		IntrFlag &= ~IR_KEY2;
		break;
		
	case D8049_KEY3:	// L[3 f[^o
		PRINTD( SUB_LOG, "[OutData KEY3] Data: %02X\n", KeyCode );
		WriteExt( KeyCode );
		IntrFlag &= ~IR_KEY3;
		break;
		
	case D8049_SIO:		// RS232CM f[^o
		PRINTD( SUB_LOG, "[OutData SIO] Data: %02X\n", SioData );
		WriteExt( SioData );
		IntrFlag &= ~IR_SIO;
	}
	Status8049 = D8049_IDLE;
}


////////////////////////////////////////////////////////////////
// CMTXe[^X擾
////////////////////////////////////////////////////////////////
int SUB6::GetCmtStatus( void )
{
	return CmtStatus;
}


////////////////////////////////////////////////////////////////
// CMTݔ?
////////////////////////////////////////////////////////////////
bool SUB6::IsCmtIntrReady( void )
{
	// ĂȂLOADOPENőOCMT READ݂ς݂IBF=L甭
	// BoostUpL̏ꍇ̓[NGA`FbN
	
	return( ( Status8049 == D8049_IDLE ) &&
			( CmtStatus == LOADOPEN ) &&
			!( IntrFlag & IR_CMTR ) &&
			!GetT0() &&
			!( vm->cmtl->IsBoostUp() && ( vm->mem->Read( 0xfa19 ) & 2 ) )	// [NGÃtOQ
			);
}


////////////////////////////////////////////////////////////////
// ǂłSAVE
////////////////////////////////////////////////////////////////
bool SUB6::DokoSave( cIni *Ini )
{
	int i;
	cSche::evinfo e;
	char stren[16];
	
	e.device = this;
	
	if( !Ini ) return false;
	
	Ini->PutEntry( "8049", NULL, "CmtStatus",	"%d",		CmtStatus );
	Ini->PutEntry( "8049", NULL, "Status8049",	"%d",		Status8049 );
	Ini->PutEntry( "8049", NULL, "IntrFlag",	"0x%08X",	IntrFlag );
	Ini->PutEntry( "8049", NULL, "KeyCode",		"0x%02X",	KeyCode );
	Ini->PutEntry( "8049", NULL, "JoyCode",		"0x%02X",	JoyCode );
	Ini->PutEntry( "8049", NULL, "CmtData",		"0x%02X",	CmtData );
	Ini->PutEntry( "8049", NULL, "SioData",		"0x%02X",	SioData );
	
	// Cxg
	int eid[] = { EID_INTCHK, EID_VECTOR, EID_DATA, 0 };
	i = 0;
	
	while( eid[i] ){
		e.id = eid[i++];
		if( vm->evsc->GetEvinfo( &e ) ){
			sprintf( stren, "Event%08X", e.id );
			Ini->PutEntry( "8049", NULL, stren, "%d %d %d %lf", e.Active ? 1 : 0, e.Period, e.Clock, e.nps );
		}
	}
	
	return true;
}


////////////////////////////////////////////////////////////////
// ǂłLOAD
////////////////////////////////////////////////////////////////
bool SUB6::DokoLoad( cIni *Ini )
{
	int st,yn,i;
	cSche::evinfo e;
	char stren[16];
	char strrs[64];
	
	e.device = this;
	
	if( !Ini ) return false;
	
	Ini->GetInt(   "8049", "CmtStatus",		&CmtStatus,		CmtStatus );
	Ini->GetInt(   "8049", "Status8049",	&Status8049,	Status8049 );
	Ini->GetInt(   "8049", "IntrFlag",		&st,		IntrFlag );	IntrFlag = st;
	Ini->GetInt(   "8049", "KeyCode",		&st,		KeyCode );	KeyCode  = st;
	Ini->GetInt(   "8049", "JoyCode",		&st,		JoyCode );	JoyCode  = st;
	Ini->GetInt(   "8049", "CmtData",		&st,		CmtData );	CmtData  = st;
	Ini->GetInt(   "8049", "SioData",		&st,		SioData );	SioData  = st;
	
	// Cxg
	int eid[] = { EID_INTCHK, EID_VECTOR, EID_DATA, 0 };
	i = 0;
	
	while( eid[i] ){
		e.id = eid[i++];
		sprintf( stren, "Event%08X", e.id );
		if( Ini->GetString( "8049", stren, strrs, "" ) ){
			sscanf( strrs,"%d %d %d %lf", &yn, &e.Period, &e.Clock, &e.nps );
			e.Active = yn ? true : false;
			if( !vm->evsc->SetEvinfo( &e ) ) return false;
		}
	}
	
	return true;
}
