#include <new>

#include "log.h"
#include "io.h"
#include "error.h"


////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
IO6::IO6( void ) : io(NULL), dl(NULL)
{
	INITARRAY( Iwait, 0 );
	INITARRAY( Owait, 0 );
}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
IO6::~IO6( void )
{
	if( dl ) delete dl;
	if( io ) delete io;
}


////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////
bool IO6::Init( int banksize )
{
	// IuWFNgm
	try{
		dl = new DeviceList;	// foCXXg
		io = new IOBus;			// I/O
		
		if( !io->Init( dl, banksize ) ) throw Error::InitFailed;
	}
	catch( std::bad_alloc ){	// new Ɏsꍇ
		Error::SetError( Error::MemAllocFailed );
		if( dl ){ delete dl; dl = NULL; }
		return false;
	}
	// O
	catch( Error::Errno i ){
		Error::SetError( i );
		return false;
	}
	
	for( int i=0; i<BANKSIZE; i++ ) Iwait[i] = Owait[i] = 0;
	
	return true;
}


////////////////////////////////////////////////////////////////
// foCXڑ
////////////////////////////////////////////////////////////////
bool IO6::Connect( IDevice* device, const IOBus::Connector* connector )
{
	return io->Connect( device, connector );
}


////////////////////////////////////////////////////////////////
// foCXؒf
////////////////////////////////////////////////////////////////
bool IO6::Disconnect( IDevice* device )
{
	return io->Disconnect( device );
}


////////////////////////////////////////////////////////////////
// IN֐
////////////////////////////////////////////////////////////////
BYTE IO6::In( int port, int *wcnt )
{
	PRINTD( IO_LOG, "[IO][In] port : %02X\n", port&0xff );
	
	if( wcnt ) (*wcnt) += Iwait[port&0xff];
	return io->In( port );
}


////////////////////////////////////////////////////////////////
// OUT֐
////////////////////////////////////////////////////////////////
void IO6::Out( int port, BYTE data, int *wcnt )
{
	PRINTD( IO_LOG, "[IO][Out] port : %02X  data : %02X\n", port&0xff, data );
	
	if( wcnt ) (*wcnt) += Owait[port&0xff];
	io->Out( port, data );
}


////////////////////////////////////////////////////////////////
// IN EFCgݒ
////////////////////////////////////////////////////////////////
void IO6::SetInWait( int port, int wait )
{
	Iwait[port&0xff] = wait;
}


////////////////////////////////////////////////////////////////
// OUTEFCgݒ
////////////////////////////////////////////////////////////////
void IO6::SetOutWait( int port, int wait )
{
	Owait[port&0xff] = wait;
}


////////////////////////////////////////////////////////////////
// IN EFCg擾
////////////////////////////////////////////////////////////////
int IO6::GetInWait( int port )
{
	return Iwait[port&0xff];
}


////////////////////////////////////////////////////////////////
// IN EFCg擾
////////////////////////////////////////////////////////////////
int IO6::GetOutWait( int port )
{
	return Owait[port&0xff];
}
