#include <new>

#include "pc6001v.h"
#include "p6el.h"
#include "error.h"

#include "breakpoint.h"
#include "common.h"
#include "cpum.h"
#include "cpus.h"
#include "debug.h"
#include "disk.h"
#include "graph.h"
#include "intr.h"
#include "keyboard.h"
#include "memory.h"
#include "osd.h"
#include "pio.h"
#include "psg.h"
#include "schedule.h"
#include "sound.h"
#include "status.h"
#include "tape.h"
#include "voice.h"
#include "vdg.h"
#include "vsurface.h"




////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
EL6::EL6( void ) : vm(NULL), cfg(NULL), sche(NULL), graph(NULL), snd(NULL), joy(NULL), staw(NULL),
	#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	regw(NULL), memw(NULL), monw(NULL),
	#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	UpDateFPSID(NULL), UDFPSCount(0), FSkipCount(0)
{}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
EL6::~EL6( void )
{
	#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	if( monw )   delete monw;
	if( memw )   delete memw;
	if( regw )   delete regw;
	#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	
	if( UpDateFPSID ) OSD_DelTimer( UpDateFPSID );
	
	if( staw )   delete staw;
	if( joy )    delete joy;
	if( snd )    delete snd;
	if( graph )  delete graph;
	if( sche )   delete sche;
	if( vm )     delete vm;
	
	// L[̓obt@j
	if( ak.Buffer ) delete [] ak.Buffer;
	
	if( GPal.colors ) delete [] GPal.colors;
}



////////////////////////////////////////////////////////////////
// Xbh֐(I[o[Ch)
////////////////////////////////////////////////////////////////
void EL6::OnThread( void *inst )
{
	EL6 *p6;
	
	p6 = STATIC_CAST( EL6 *, inst );	// g̃IuWFNg|C^擾
	
	#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	if( p6->cfg->GetMonDisp() ){
	// j^[h
		while( !this->cThread::IsCancel() ){
			// ʍXV
			if( p6->sche->IsScreenUpdate() ) p6->ScreenUpdate();
			p6->Wait();		// EFCg
		}
	}else
	// ʏ탂[h
		if( p6->vm->bp->ExistBreakPoint() ){
			// u[N|Cg
			while( !this->cThread::IsCancel() ){
				p6->Emu();		// 1ߎs
				
				if( p6->vm->evsc->IsVSYNC() ){
					p6->vm->key->ScanMatrix();	// L[}gNXXL
					
					// TEhXV
					p6->SoundUpdate( 0 );
					// ʍXV
					if( p6->sche->IsScreenUpdate() ) p6->ScreenUpdate();
					p6->Wait();		// EFCg
				}
				
				// u[N|Cg`FbN
				if( p6->vm->bp->CheckBreakPoint( BPoint::BP_PC, p6->vm->cpum->GetPC(), 0, NULL ) || p6->vm->bp->IsReqBreak() ){
					this->cThread::Cancel();	// XbhItOĂ
					p6->vm->bp->ResetBreak();
					OSD_PushEvent( EV_DEBUGMODEBP, p6->vm->cpum->GetPC() );
				}
			}
		}else
	#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	{	// ʏs
		while( !this->cThread::IsCancel() ){
			// L[}gNXXL
			bool matchg = p6->vm->key->ScanMatrix();
			
			// vCL^
			if( REPLAY::GetStatus() == REP_RECORD )
				REPLAY::ReplayWriteFrame( p6->vm->key->GetMatrix2(), matchg );
			
			// vCĐ
			if( REPLAY::GetStatus() == REP_REPLAY ){
				REPLAY::ReplayReadFrame( p6->vm->key->GetMatrix() );
			}
			
			p6->EmuVSYNC();			// 1ʕs
			
			
			// rfILv`?
			if( AVI6::IsAVI() ){
				// rfILv`Ȃ炱AVIۑ
				// TEhXV
				p6->SoundUpdate( 0, AVI6::GetAudioBuffer() );
				// ʍXV
				bool update = p6->ScreenUpdate();
				// ʍXVĂ AVI1ʕۑ
				if( update ) AVI6::AVIWriteFrame( p6->graph->GetSubBuffer() );
			}else{
				// rfILv`łȂȂʏ̍XV
				// TEhXV
				p6->SoundUpdate( 0 );
				// ʍXV
				// ʍXV}ĂʍXV
				// m[EFCg̎FPSςȂ悤ɂ
				if( p6->sche->IsScreenUpdate() ) p6->ScreenUpdate();
			}
			
			// L[
			if( IsAutoKey() ){
				BYTE key = GetAutoKey();
				if( key ) p6->vm->cpus->ReqKeyIntr( 0, key );
			}
			
			p6->Wait();		// EFCg
		}
	}
}


////////////////////////////////////////////////////////////////
// Wait
////////////////////////////////////////////////////////////////
void EL6::Wait( void )
{
	if( sche->GetWaitEnable() && (!cfg->GetTurboTAPE() || (vm->cpus->GetCmtStatus() == CMTCLOSE)) )
		sche->VWait();
	vm->evsc->ReVSYNC();
}


////////////////////////////////////////////////////////////////
// 1ߎs
////////////////////////////////////////////////////////////////
int EL6::Emu( void )
{
	int st = vm->Emu();			// VM 1ߎs
	vm->evsc->Update( st );		// CxgXV
	sche->Update( st );
	
	return st;
}


////////////////////////////////////////////////////////////////
// 1ʕs
////////////////////////////////////////////////////////////////
int EL6::EmuVSYNC( void )
{
	int state = 0;
	
	// VSYNC܂ŌJԂ
	while( !vm->evsc->IsVSYNC() ){
		int st = vm->Emu();		// VM 1ߎs
		vm->evsc->Update( st );	// CxgXV
		sche->Update( st );
		state += st;
	}
	
	return state;
}


#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
////////////////////////////////////////////////////////////////
// wXe[gs
////////////////////////////////////////////////////////////////
void EL6::Exec( int st )
{
	int State = st;
	
	while( State > 0 ) State -= Emu();
}


////////////////////////////////////////////////////////////////
// j^[hؑւ
////////////////////////////////////////////////////////////////
bool EL6::ToggleMonitor( void )
{
	// VM~
	Stop();
	
	// j^EBhE\Ԑ؊
	cfg->SetMonDisp( cfg->GetMonDisp() ? false : true );
	
	// u[N|Cg̏NA
	vm->bp->ClearStatus();
	
	// XN[TCYύX
	graph->ResizeScreen();
	
	// VMĊJ
	Start();
	
	return cfg->GetMonDisp();
}
#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //


////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////
bool EL6::Init( const CFG6 *config )
{
	// G[bZ[Wl
	Error::SetError( Error::InitFailed );
	
	if( !config ) return false;
	cfg = CONST_CAST( CFG6 *, config );
	
	// pbg
	GPal.colors = new COLOR24[256];
	if( !GPal.colors ) return false;
	GPal.ncols = 256;
	
	for( int i=0; i<128; i++ ){
		cfg->GetColor( i, &GPal.colors[i] );
		
		// XLCpJ[ݒ
		GPal.colors[i+128].r = ( GPal.colors[i].r * cfg->GetScanLineBr() ) / 100;
		GPal.colors[i+128].g = ( GPal.colors[i].g * cfg->GetScanLineBr() ) / 100;
		GPal.colors[i+128].b = ( GPal.colors[i].b * cfg->GetScanLineBr() ) / 100;
	}
	
	// @ VMm
	switch( cfg->GetModel() ){
	case 62: vm = new VM62( this ); break;
	case 66: vm = new VM66( this ); break;
	default: vm = new VM60( this );
	}
	// VM
	if( !vm || !vm->Init( cfg ) ) return false;
	
	// eIuWFNgm
	sche   = new SCH6( vm->GetCPUClock() * cfg->GetOverClock() / 100 );	// XPW[
	snd    = new SND6;											// TEh
	graph  = new DSP6( vm );									// ʕ`
	joy    = new JOY6;											// WCXeBbN
	staw   = new cWndStat( vm );								// Xe[^Xo[
	
	if( !sche || !snd || !graph || !staw || !joy ) return false;
	
	#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	regw   = new cWndReg( vm, DEV_ID("REGW") );					// WX^EBhE
	memw   = new cWndMem( vm, DEV_ID("MEMW") );					// EBhE
	monw   = new cWndMon( vm, DEV_ID("MONW") );					// j^EBhE
	
	if( !regw || !memw || !monw ) return false;
	#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	
	// TEh -----
	if( !snd->Init( this, EL6::StreamUpdate, cfg->GetSampleRate(), cfg->GetSoundBuffer() ) ) return false;
	snd->SetVolume( cfg->GetMasterVol() );
	
	// ʕ` -----
	if( !graph->Init( cfg->GetScrBpp(), cfg->GetScanLineBr(), &GPal ) ) return false;
	graph->SetIcon( cfg->GetModel() );	// ACRݒ
	
	// WCXeBbN -----
	if( !joy->Init() ) return false;
	
	// Xe[^Xo[ -----
	if( !staw->Init( graph->ScreenX(), cfg->GetFddNum() ) ) return false;
	
	#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	// WX^EBhE@-----
	if( !regw->Init() ) return false;
	
	// EBhE@-----
	if( !memw->Init() ) return false;
	
	// j^EBhE@-----
	if( !monw->Init() ) return false;
	#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
	
	// FPS\^C}ݒ
	if( !SetFPSTimer( 1 ) ){
		Stop();
		return false;
	}
	
	// rfILv` -----
	if( !AVI6::Init() ) return false;
	
	// vC -----
	if( !REPLAY::Init( vm->key->GetMatrixSize() ) ) return false;
	
	// XN[TCYύX
	graph->ResizeScreen();
	
	
	// TAPE}
	if( *(cfg->GetTapeFile()) ) TapeMount( cfg->GetTapeFile() );
	// hCu1,2DISK}
	if( *(cfg->GetDiskFile( 1 )) ) DiskMount( 0, cfg->GetDiskFile( 1 ) );
	if( *(cfg->GetDiskFile( 2 )) ) DiskMount( 1, cfg->GetDiskFile( 2 ) );
	
	
	vm->Reset();
	
	
	// Xg[ڑ
	snd->ConnectStream( STATIC_CAST( SndDev *, vm->psg ) );						// PSG
	snd->ConnectStream( STATIC_CAST( SndDev *, vm->cmtl ) );					// CMT(LOAD)
	if( vm->voice ) snd->ConnectStream( STATIC_CAST( SndDev *, vm->voice ) );	// 
	
	// G[Ȃ
	Error::SetError( Error::NoError );
	
	return true;
}


////////////////////////////////////////////////////////////////
// Jn
////////////////////////////////////////////////////////////////
bool EL6::Start( void )
{
	FSkipCount = 0;
	
	// Xbh
	if( !this->cThread::BeginThread( this ) ) return false;
	
	sche->Start();
	snd->Play();
	
	return true;
}


////////////////////////////////////////////////////////////////
// ~
////////////////////////////////////////////////////////////////
void EL6::Stop( void )
{
	this->cThread::Cancel();	// XbhItOĂ
	this->cThread::Waiting();	// XbhI܂ő҂
	
	snd->Pause();
	sche->Stop();
}


////////////////////////////////////////////////////////////////
// Cxg[v
////////////////////////////////////////////////////////////////
EL6::ReturnCode EL6::EventLoop( void )
{
	Event event;
	
	while( OSD_GetEvent( &event ) ){
		switch( event.type ){
		case EV_FPSUPDATE:		// FPS\
			{
			char str[256];
			sprintf( str, "%s (%3d%%  %2d/%2d fps)", cfg->GetCaption(), sche->GetRatio(), event.fps.fps, FRAMERATE );
			OSD_SetWindowCaption( graph->GetWindowHandle(), str );
			}
			break;
			
		case EV_KEYDOWN:		// L[
			#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
			// j^[h?
			if( cfg->GetMonDisp() ){
				monw->KeyIn( event.key.sym, event.key.mod & KVM_SHIFT, event.key.unicode );
				break;
			}else
			#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
			// e@\L[`FbN
			if( CheckFuncKey( event.key.sym,
							  event.key.mod & KVM_ALT  ? true : false,
							  event.key.mod & KVM_META ? true : false ) )
				break;
			// vCĐ or L[͎słȂ
			if( REPLAY::GetStatus() != REP_REPLAY && !IsAutoKey() )
				// L[}gNXXV(L[)
				vm->key->UpdateMatrixKey( event.key.sym, true );
			break;
			
		case EV_KEYUP:
			// vCĐ or L[͎słȂ
			if( REPLAY::GetStatus() != REP_REPLAY && !IsAutoKey() )
				// L[}gNXXV(L[)
				vm->key->UpdateMatrixKey( event.key.sym, false );
			break;
			
		case EV_JOYAXISMOTION:
		case EV_JOYBUTTONDOWN:
		case EV_JOYBUTTONUP:
			// vCĐ or L[͎słȂ
			if( REPLAY::GetStatus() != REP_REPLAY && !IsAutoKey() )
				// L[}gNXXV(WCXeBbN)
				vm->key->UpdateMatrixJoy( joy->GetJoyState( 0 ), joy->GetJoyState( 1 ) );
			break;
			
		#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
		case EV_DEBUGMODEBP:	// j^[hύX(u[N|CgB)
			monw->BreakIn( event.bp.addr );		// u[N|Cg̏\
			ToggleMonitor();					// j^[hύX
			
			break;
			
		#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
		case EV_MOUSEBUTTONUP:	// }EX{^NbN
			if( ( event.mousebt.button == MBT_RIGHT ) && ( event.mousebt.state == false ) ){
				Stop();
				
				// |bvAbvj[\
				ShowPopupMenu( event.mousebt.x, event.mousebt.y );
				
				Start();
			}
			break;
			
		case EV_QUIT:			// I
			if( cfg->GetCkQuit() )
				if( OSD_Message( MSG_QUIT, MSG_QUITC, OSDM_YESNO | OSDM_ICONQUESTION ) != OSDR_YES )
					break;
			return Quit;
			
		case EV_RESTART:		// ċN
			return Restart;
			
		case EV_DOKOLOAD:		// ǂłLOAD
			return Dokoload;
			
		case EV_REPLAY:			// vCĐ
			return Replay;
			
		default:
			break;
		}
		
		// G[
		switch( Error::GetError() ){
		case Error::NoError:
			break;
			
		default:
			OSD_Message( (char *)Error::GetErrorText(), MSERR_ERROR, OSDR_OK | OSDM_ICONERROR );
			Error::SetError( Error::NoError );
		}
	}
	return Quit;
}


////////////////////////////////////////////////////////////////
// e@\L[`FbN
////////////////////////////////////////////////////////////////
bool EL6::CheckFuncKey( int kcode, bool OnALT, bool OnMETA )
{
	switch( kcode ){	// L[R[h
	case KVC_F6:		// j^[hύX or XN[[hύX
		// rfILv`͖
		if( AVI6::IsAVI() ) return false;
		
		if( OnALT ){
			Stop();
			cfg->SetFullScreen( cfg->GetFullScreen() ? false : true );
			graph->ResizeScreen();	// XN[TCYύX
			Start();
		}else{
			#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
			ToggleMonitor();
			#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
		}
		break;
		
	case KVC_F7:			// XLC[hύX
		// rfILv`͖
		if( AVI6::IsAVI() ) return false;
		
		Stop();
		if( OnALT ){
			cfg->SetDispNTSC( cfg->GetDispNTSC() ? false : true );
		}else{
			cfg->SetScanLine( cfg->GetScanLine() ? false : true );
		}
		graph->ResizeScreen();	// XN[TCYύX
		Start();
		break;
		
	case KVC_F8:			// [h4J[ύX or Xe[^Xo[\ԕύX
		if( OnALT ){
			Stop();
			cfg->SetStatDisp( cfg->GetStatDisp() ? false : true );
			graph->ResizeScreen();	// XN[TCYύX
			Start();
		}else{
			int c = vm->vdg->GetMode4Color();
			if( ++c > 4 ) c = 0;
			vm->vdg->SetMode4Color( c );
		}
		break;
		
	case KVC_F10:			// WaitLύX
		if( OnALT ){
		}else{
			sche->SetWaitEnable( sche->GetWaitEnable() ? false : true );
		}
		break;
		
	case KVC_F11:			// Zbg or ċN
		if( OnALT ){
			OSD_PushEvent( EV_RESTART );
		}else{
			vm->Reset();
		}
		break;
		
	case KVC_F12:			// XibvVbg
		if( OnALT ){
		}else{
			graph->SnapShot( cfg->GetImgPath() );
		}
		break;
		
	default:				// ǂłȂ
		return false;
	}
	return true;
}


////////////////////////////////////////////////////////////////
// ʍXV
////////////////////////////////////////////////////////////////
bool EL6::ScreenUpdate( void )
{
	// t[XLbv`FbN
	if( ++FSkipCount > cfg->GetFrameSkip() ){
		// Xe[^Xo[XV
		staw->SetReplayStatus( REPLAY::GetStatus() );	// vCXe[^X
		vm->vdg->UpdateBackBuf();	// obNobt@XV
		graph->DrawScreen();		// ʕ`
		
		#ifndef NOMONITOR	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
		if( cfg->GetMonDisp() ){
			regw->Update();
			memw->Update();
			monw->Update();
		}
		#endif				// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ //
		
		UDFPSCount++;
		FSkipCount = 0;
		
		return true;
	}
	return false;
}


////////////////////////////////////////////////////////////////
// TEhXV
//
// :	samples		XVTv(-1:cobt@S 0:NbN)
//			exbuf		Oobt@
// Ԓl:	int			XVTv
////////////////////////////////////////////////////////////////
int EL6::SoundUpdate( int samples, cRing *exbuf )
{
	// PSGXV
	vm->psg->SoundUpdate( samples );
	
	// CMT(LOAD)XV
	vm->cmtl->SoundUpdate( vm->psg->SndDev::cRing::ReadySize() );
	
	// XV
	if( vm->voice ) vm->voice->SoundUpdate( vm->psg->SndDev::cRing::ReadySize() );
	
	// TEhobt@XV
	return snd->PreUpdate( vm->psg->SndDev::cRing::ReadySize(), exbuf );
}


////////////////////////////////////////////////////////////////
// Xg[XV R[obN֐
//
// :	userdata	R[obN֐ɓnf[^(gւ̃IuWFNg|C^)
//			stream		Xg[݃obt@ւ̃|C^
//			len			obt@TCY(oCgP)
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void EL6::StreamUpdate( void *userdata, BYTE *stream, int len )
{
	EL6 *p6 = STATIC_CAST( EL6 *, userdata );	// g̃IuWFNg|C^擾
	
	// TEhobt@XV
	//  TvȂ΂Œǉ
	//  rfILv`͖
	int addsam = len/sizeof(int16_t) - p6->snd->cRing::ReadySize();
	
	p6->snd->Update( stream, min( len/sizeof(int16_t), p6->snd->cRing::ReadySize() ) );
	
	if( addsam > 0 && !p6->AVI6::IsAVI() ){
		p6->SoundUpdate( addsam );
		p6->snd->Update( stream, addsam );
	}
}


////////////////////////////////////////////////////////////////
// FPS\^C} R[obN֐
////////////////////////////////////////////////////////////////
DWORD EL6::UpDateFPS( DWORD interval, void *obj )
{
	EL6 *p6 = STATIC_CAST( EL6 *, obj );	// g̃IuWFNg|C^擾
	
	OSD_PushEvent( EV_FPSUPDATE, p6->UDFPSCount );
	p6->UDFPSCount = 0;
	
	return interval;
}


////////////////////////////////////////////////////////////////
// FPS\^C}ݒ
////////////////////////////////////////////////////////////////
bool EL6::SetFPSTimer( int fps )
{
	// ^C}ғȂ~
	if( UpDateFPSID ){
		OSD_DelTimer( UpDateFPSID );
		UpDateFPSID = NULL;
	}
	
	// ^C}ݒ
	if( fps > 0 )
		UpDateFPSID = OSD_AddTimer( 1000/fps, EL6::UpDateFPS, this );
	
	return UpDateFPSID ? true : false;
}


////////////////////////////////////////////////////////////////
// L[͎s?
//
// :	Ȃ
// Ԓl:	bool	true:s false:słȂ
////////////////////////////////////////////////////////////////
bool EL6::IsAutoKey( void )
{
	return ak.Num ? true : false;
}


////////////////////////////////////////////////////////////////
// L[1擾
//   (VSYNC=1/60secɌĂ΂)
//
// :	Ȃ
// Ԓl:	BYTE	P6̃L[R[h
////////////////////////////////////////////////////////////////
char EL6::GetAutoKey( void )
{
	// [OFF҂
	if( ak.Relay ){
		if( vm->cmtl->IsRelay() ) return 0;
		else                 ak.Relay = false;
	}
	
	// ҂?
	if( ak.Wait > 0 ){
		ak.Wait--;
		return 0;
	}
	
	// ̕擾
	if( ak.Num-- > 0 ){
		BYTE dat = ak.Buffer[ak.Seek++];
		switch( dat ){
		case 0x17:	// '\w' EFCgݒ
			if( ak.Num-- > 0 ) ak.Wait += (int)((BYTE)ak.Buffer[ak.Seek++]);	// ҂񐔐ݒ
			dat = 0;
			break;
			
		case 0x0a:	// '\r' [OFF҂
			ak.Relay = true;
			dat = 0x0d;
		case 0x0d:	// '\n' s?
			ak.Wait = 9;	// ҂9(=150msec)
			break;
			
		default:	// ʂ̕
			ak.Wait = 0;	// ҂Ȃ
		}
		return dat;
	}else{
		// obt@ȂI
		delete [] ak.Buffer;
		ak.Buffer = NULL;
	}
	return 0;
}


////////////////////////////////////////////////////////////////
// L[͕ݒ
//
// :	str		
//			num		̒
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool EL6::SetAutoKey( char *str, int num )
{
	// L[obt@U폜
	
	// L[obt@m
	if( ak.Buffer ) delete [] ak.Buffer;
	ak.Buffer = new char[num+1];
	if( !ak.Buffer ){
		ak.Num = 0;
		return false;
	}
	
	// Rs[
	strncpy( ak.Buffer, str, num );
	
	ak.Num    = num;	// c蕶
	ak.Wait   = 60;		// ҂񐔃JE^(1sec)
	ak.Relay  = false;	// [XCb`OFF҂tO
	ak.Seek   = 0;		// Ǎ݃|C^
	
	return true;
}


////////////////////////////////////////////////////////////////
// L[͕ݒ(t@C)
//
// :	filepath	̓t@Cւ̃pX
// Ԓl:	bool		true: false:s
////////////////////////////////////////////////////////////////
bool EL6::SetAutoKeyFile( char *filepath )
{
	FILE *fp;
	char lbuf[1024];
	
	fp = FOPENEN( filepath, "rb" );
	if( !fp ) return false;
	
	fseek( fp, 0, SEEK_END );
	int tsize = ftell( fp ) + 1;
	fseek( fp, 0, SEEK_SET );
	
	// L[obt@m
	if( ak.Buffer ) delete [] ak.Buffer;
	ak.Buffer = new char[tsize+1];
	if( !ak.Buffer ){
		fclose( fp );
		ak.Num = 0;
		return false;
	}
	ZeroMemory( ak.Buffer, sizeof(ak.Buffer) );
	
	// Ǎ
	// f[^Ȃ܂ŌJԂ
	while( fgets( lbuf, 1024, fp ) ){
		Sjis2P6( lbuf, lbuf );	// SJIS -> P6
		strcat( ak.Buffer, lbuf );
	}
	fclose( fp );
	tsize = strlen( ak.Buffer );
	
	ak.Num    = tsize;	// c蕶
	ak.Wait   = 60;		// ҂񐔃JE^(1sec)
	ak.Relay  = false;	// [XCb`OFF҂tO
	ak.Seek   = 0;		// Ǎ݃|C^
	
	return true;
}


////////////////////////////////////////////////////////////////
// ǂłSAVE
//
// :	filename	t@C
// Ԓl:	bool		true: false:s
////////////////////////////////////////////////////////////////
bool EL6::DokoDemoSave( char *filename )
{
	cIni *Ini = NULL;
	
	// Ƃ肠G[ݒ
	Error::SetError( Error::DokoWriteFailed );
	try{
		FILE *fp = FOPENEN( filename, "wt" );
		if( !fp ) throw Error::DokoWriteFailed;
		// ^Cgso͂ĈU
		fprintf( fp, MSDOKO_TITLE );
		fclose( fp );
		
		// ǂłSAVEt@CJ
		Ini = new cIni();
		if( !Ini->Init( filename ) ) throw Error::DokoWriteFailed;
		
		if( !cfg->DokoSave( Ini ) ||
			!vm->evsc->DokoSave( Ini ) ||
			!vm->psg->DokoSave( Ini ) ||
			!vm->disk->DokoSave( Ini ) ||
			!vm->vdg->DokoSave( Ini ) ||
			!vm->intr->DokoSave( Ini ) ||
			!vm->key->DokoSave( Ini ) ||
			!vm->cpus->DokoSave( Ini ) ||
			!vm->pio->DokoSave( Ini ) ||
			!vm->cpum->DokoSave( Ini ) ||
			!vm->mem->DokoSave( Ini ) ||
			!vm->cmtl->DokoSave( Ini )
		) throw Error::GetError();
	}
	catch( std::bad_alloc ){	// new Ɏsꍇ
		Error::SetError( Error::MemAllocFailed );
		return false;
	}
	catch( Error::Errno i ){	// O
		Error::SetError( i );
		if( Ini ) delete Ini;
		return false;
	}
	
	Ini->Write();
	
	delete Ini;
	
	// ̂ŃG[Ȃ
	Error::SetError( Error::NoError );
	
	return true;
}


////////////////////////////////////////////////////////////////
// ǂłLOAD
//
// :	filename	t@C
// Ԓl:	bool		true: false:s
////////////////////////////////////////////////////////////////
bool EL6::DokoDemoLoad( char *filename )
{
	cIni *Ini = NULL;
	
	// Ƃ肠G[ݒ
	Error::SetError( Error::DokoReadFailed );
	try{
		// ǂłLOADt@CJ
		Ini = new cIni();
		if( !Ini->Init( filename ) ) throw Error::DokoReadFailed;
		cfg->SetDokoFile( filename );
		
		// ܂PC6001Ṽo[WmF
		if( !cfg->DokoLoad( Ini ) ) throw Error::GetError();
		
		vm->Reset();
		
		if(	!vm->evsc->DokoLoad( Ini ) ||
			!vm->psg->DokoLoad( Ini ) ||
			!vm->disk->DokoLoad( Ini ) ||
			!vm->vdg->DokoLoad( Ini ) ||
			!vm->intr->DokoLoad( Ini ) ||
			!vm->key->DokoLoad( Ini ) ||
			!vm->cpus->DokoLoad( Ini ) ||
			!vm->pio->DokoLoad( Ini ) ||
			!vm->cpum->DokoLoad( Ini ) ||
			!vm->mem->DokoLoad( Ini ) ||
			!vm->cmtl->DokoLoad( Ini )
		) throw Error::GetError();
		
		// fBXNhCuɂăXN[TCYύX
		if( !staw->Init( -1, vm->disk->GetDrives() ) ) throw Error::GetError();
		if( !graph->ResizeScreen() ) throw Error::GetError();
	}
	catch( std::bad_alloc ){	// new Ɏsꍇ
		Error::SetError( Error::MemAllocFailed );
		return false;
	}
	catch( Error::Errno i ){	// O
		Error::SetError( i );
		if( Ini ) delete Ini;
		return false;
	}
	
	delete Ini;
	cfg->SetDokoFile( "" );
	
	// ̂ŃG[Ȃ
	Error::SetError( Error::NoError );
	
	return true;
}


////////////////////////////////////////////////////////////////
// ǂłLOADt@C@햼Ǎ
//
// :	filename	t@C
// Ԓl:	int			@햼(60,62,66)
////////////////////////////////////////////////////////////////
int EL6::GetDokoModel( char *filename )
{
	cIni *Ini = NULL;
	int st;
	
	try{
		// ǂłLOADt@CJ
		Ini = new cIni();
		if( !Ini->Init( filename ) ) throw Error::DokoReadFailed;
		
		// @擾
		Ini->GetInt( "GLOBAL", "P6Model",	&st, 0 );
	}
	catch( std::bad_alloc ){	// new Ɏsꍇ
		Error::SetError( Error::MemAllocFailed );
		return 0;
	}
	catch( Error::Errno i ){	// O
		Error::SetError( i );
		if( Ini ) delete Ini;
		return 0;
	}
	
	delete Ini;
	
	return st;
}







////////////////////////////////////////////////////////////////
// TAPE }Eg
//
// :	filename	t@C
// Ԓl:	bool		true: false:s
////////////////////////////////////////////////////////////////
bool EL6::TapeMount( char *filename )
{
	if( !vm->cmtl->Mount( filename ) ){
		staw->TapeUnmount();
		return false;
	}
	
	staw->TapeMount( *vm->cmtl->GetName() ? (char *)vm->cmtl->GetName() : (char *)OSD_GetFileNamePart( filename ),
					  vm->cmtl->IsAutoStart(), vm->cmtl->GetSize() );
	return true;
}


////////////////////////////////////////////////////////////////
// TAPE A}Eg
//
// :	Ȃ
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void EL6::TapeUnmount( void )
{
	vm->cmtl->Unmount();
	staw->TapeUnmount();
}


////////////////////////////////////////////////////////////////
// DISK }Eg
//
// :	drv			hCuԍ
//			filename	t@C
// Ԓl:	bool		true: false:s
////////////////////////////////////////////////////////////////
bool EL6::DiskMount( int drv, char *filename )
{
	if( !vm->disk->Mount( drv, filename ) ){
		staw->DiskUnmount( drv );
		return false;
	}
	
	staw->DiskMount( drv, *vm->disk->GetName( drv ) ? (char *)vm->disk->GetName( drv ) : (char *)OSD_GetFileNamePart( filename ),
					  vm->disk->IsSystem( drv ), vm->disk->IsProtect( drv ) );
	return true;
}


////////////////////////////////////////////////////////////////
// DISK A}Eg
//
// :	drv			hCuԍ
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void EL6::DiskUnmount( int drv )
{
	vm->disk->Unmount( drv );
	staw->DiskUnmount( drv );
}







////////////////////////////////////////////////////////////////
// vCۑJn
//
// :	filename	t@C
// Ԓl:	bool		true: false:s
////////////////////////////////////////////////////////////////
bool EL6::ReplayStartRec( char *filename )
{
	return REPLAY::StartRecord( filename );
}


////////////////////////////////////////////////////////////////
// vCۑ~
//
// :	Ȃ
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void EL6::ReplayStopRec( void )
{
	REPLAY::StopRecord();
}


////////////////////////////////////////////////////////////////
// vCĐJn
//
// :	filename	t@C
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void EL6::ReplayStartPlay( char *filename )
{
	cfg->SetModel( GetDokoModel( filename ) );
	cfg->SetDokoFile( filename );
	OSD_PushEvent( EV_REPLAY );
}


////////////////////////////////////////////////////////////////
// vCĐ~
//
// :	Ȃ
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void EL6::ReplayStopPlay( void )
{
	REPLAY::StopReplay();
}

