#include "pc6001v.h"
#include "log.h"
#include "p6t2.h"
#include "common.h"
#include "osd.h"


// P6T`tH[}bgVer.2
//  {Iɂ́ux^C[W+tb^+x^C[WTCY(4byte)vƂ\
//  tb^ɂDATAubN񂪊܂܂B
//  DATAubN͖؂Ƃ1ƐB
//  BASICȂǁuwb_{f[^vƂ\̏ꍇCDATAubN2ƂȂ邪
//  IDԍ𓯈ɂ邱ƂŊ֘AtĊǗłB
//
// [tb^]
//  header (2byte) : "P6"
//  ver    (1byte) : o[W
//  dbnum  (1byte) : ܂܂DATAubN(255܂)
//  start  (1byte) : I[gX^[gtO(0: 1:L)
//  basic  (1byte) : BASIC[h(PC-6001̏ꍇ͖Ӗ)
//  page   (1byte) : y[W
//  askey  (2byte) : I[gX^[gR}h
//  ...R}hꍇ͂̌askey
//  exhead (2byte) : gTCY(64KB܂)
//  ...g񂪂ꍇ͂̌exhead
//
// [DATAubN]
//  header (2byte) : "TI"
//  id     (1byte) : IDԍ(DATAubN֘At)
//  name  (16byte) : f[^(15+'00H')
//  baud   (2byte) : {[[g(600/1200)
//  stime  (2byte) : ̎(ms)
//  ptime  (2byte) : ҁ[̎(ms)
//  offset (4byte) : x^C[W擪̃ItZbg
//  size   (4byte) : f[^TCY



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
cP6DATA::cP6DATA( void ) : 	Data(NULL), next(NULL), before(NULL) {}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
cP6DATA::~cP6DATA( void )
{
	if( Data ) delete [] Data;
	if( next ) delete next;
	if( before ) before->next = NULL;
}


////////////////////////////////////////////////////////////////
// 擪ubNւ̃|C^Ԃ
////////////////////////////////////////////////////////////////
cP6DATA *cP6DATA::TopBlock( void )
{
	cP6DATA *top = this;
	while( top->before ) top = top->before;
	
	return top;
}


////////////////////////////////////////////////////////////////
// ubNւ̃|C^Ԃ
////////////////////////////////////////////////////////////////
cP6DATA *cP6DATA::LastBlock( void )
{
	cP6DATA *last = this;
	while( last->next ) last = last->next;
	
	return last;
}


////////////////////////////////////////////////////////////////
// VKDATAubNǉ
////////////////////////////////////////////////////////////////
cP6DATA *cP6DATA::New( void )
{
	LastBlock()->next = new cP6DATA;
	LastBlock()->before = this;
	
	return LastBlock();
}


////////////////////////////////////////////////////////////////
// Rs[쐬
////////////////////////////////////////////////////////////////
cP6DATA *cP6DATA::Clone( void )
{
	cP6DATA *newb = new cP6DATA;
	*newb = *this;
	newb->next = newb->before = NULL;
	
	if( this->Data ){
		newb->Data = new BYTE[Info.DNum];
		memcpy( newb->Data, this->Data, Info.DNum );
	}
	
	return newb;
}


////////////////////////////////////////////////////////////////
// SRs[쐬
////////////////////////////////////////////////////////////////
cP6DATA *cP6DATA::Clones( void )
{
	cP6DATA *oldb = TopBlock();
	cP6DATA *newb = TopBlock()->Clone();
	
	while( oldb->next ){
		oldb = oldb->next;
		newb->next         = oldb->Clone();
		newb->next->before = newb;
		newb = newb->LastBlock();
	}
	
	return newb->TopBlock();
}


////////////////////////////////////////////////////////////////
// f[^Zbg
////////////////////////////////////////////////////////////////
int cP6DATA::SetData( FILE *fp, int num )
{
	// Zbgς݂ȂJ
	if( Data ) delete Data;
	
	// m
	Data = new BYTE[num];
	if( Data ) Info.DNum = num;
	else       return 0;
	
	Info.Offset = ftell( fp );
	
	if( fp ) fread( Data, sizeof(BYTE), num, fp );
	else     ZeroMemory( Data, num );
	
	return Info.DNum;
}


////////////////////////////////////////////////////////////////
// ,ҁ[ԃZbg
////////////////////////////////////////////////////////////////
void cP6DATA::SetPeriod( int stime, int ptime )
{
	PRINTD( P6T2_LOG, "[cP6DATA][SetPeriod] s:%d p:%d\n", stime, ptime );
	
	Info.STime = stime;	// ̎(ms)Zbg
	Info.PTime = ptime;	// ҁ[̎(ms)Zbg
}


////////////////////////////////////////////////////////////////
// P6T ubN擾
////////////////////////////////////////////////////////////////
void cP6DATA::GetInfo( P6TBLKINFO *info )
{
	info->STime  = Info.STime;	// ̎(ms)
	info->PTime  = Info.PTime;	// ҁ[̎(ms)
	info->Offset = Info.Offset;	// x^C[W擪̃ItZbg
	info->DNum   = Info.DNum;	// f[^TCY
}


////////////////////////////////////////////////////////////////
// 1ByteǍ
////////////////////////////////////////////////////////////////
BYTE cP6DATA::Read( int num )
{
	if( !Data || num > Info.DNum ) return 0;
	return Data[num];
}


////////////////////////////////////////////////////////////////
// t@Cɏ(f[^)
////////////////////////////////////////////////////////////////
int cP6DATA::Writefd( FILE *fp )
{
	Info.Offset = ftell( fp );
	
	return fwrite( Data, sizeof(BYTE), Info.DNum, fp );
}



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
cP6PART::cP6PART( void ) : 	ID(0), Baud(1200), Data(NULL), next(NULL), before(NULL)
{
	INITARRAY( Name, 0 );
}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
cP6PART::~cP6PART( void )
{
	if( Data ) delete Data;
	if( next ) delete next;
	if( before ) before->next = NULL;
}


////////////////////////////////////////////////////////////////
// 擪PARTւ̃|C^Ԃ
////////////////////////////////////////////////////////////////
cP6PART *cP6PART::TopPart( void )
{
	cP6PART *top = this;
	while( top->before ) top = top->before;
	
	return top;
}


////////////////////////////////////////////////////////////////
// PARTւ̃|C^Ԃ
////////////////////////////////////////////////////////////////
cP6PART *cP6PART::LastPart( void )
{
	cP6PART *last = this;
	while( last->next ) last = last->next;
	
	return last;
}


////////////////////////////////////////////////////////////////
// CӂPARTւ̃|C^Ԃ
////////////////////////////////////////////////////////////////
cP6PART *cP6PART::Part( int num )
{
	cP6PART *part = TopPart();
	
	for( int i=0; i< num; i++ ){
		if( part->next ) part = part->next;
		else             break;
	}
	
	return part;
}


////////////////////////////////////////////////////////////////
// IDԍio[
////////////////////////////////////////////////////////////////
BYTE cP6PART::Renumber( void )
{
	cP6PART *part = TopPart();
	
	part->ID = 0;
	while( part->next ){
		part->next->ID = part->ID + 1;
		part = part->next;
	}
	
	return LastPart()->ID;
}


////////////////////////////////////////////////////////////////
// VKPARTǉ
////////////////////////////////////////////////////////////////
cP6PART *cP6PART::New( void )
{
	LastPart()->next = new cP6PART;
	LastPart()->before = this;
	
	return LastPart();
}


////////////////////////////////////////////////////////////////
// Rs[쐬
////////////////////////////////////////////////////////////////
cP6PART *cP6PART::Clone( void )
{
	cP6PART *newp = new cP6PART;
	*newp = *this;
	newp->next = newp->before = NULL;
	
	if( this->Data ) newp->Data = this->Data->Clones();
	
	return newp;
}


////////////////////////////////////////////////////////////////
// PARTA
////////////////////////////////////////////////////////////////
cP6PART *cP6PART::Link( cP6PART *src )
{
	cP6PART *last = LastPart();
	
	last->next  = src;
	src->before = last;
	
	Renumber();
	
	return src;
}


////////////////////////////////////////////////////////////////
// VKDATAubNǉ
////////////////////////////////////////////////////////////////
cP6DATA *cP6PART::NewBlock( void )
{
	if( Data ) return Data->New();
	else       return Data = new cP6DATA;
}


////////////////////////////////////////////////////////////////
// SDATAubN擾
////////////////////////////////////////////////////////////////
int cP6PART::GetBlocks( void )
{
	int num = 0;
	cP6DATA *data = Data;
	
	do{
		num ++;
		data = data->Next();
	}while( data );
	
	return num;
}


////////////////////////////////////////////////////////////////
// PARTTCY擾
////////////////////////////////////////////////////////////////
int cP6PART::GetSize( void )
{
	int size = 0;
	
	cP6DATA *data = Data;
	while( data ){
		size += data->GetSize();
		data  = data->Next();
	}
	
	return size;
}

////////////////////////////////////////////////////////////////
// f[^ݒ
////////////////////////////////////////////////////////////////
int cP6PART::SetName( const char *name )
{
	ZeroMemory( Name, sizeof(Name) );
	strncpy( Name, name, sizeof(Name)-1 );
	
	return strlen( Name );
}


////////////////////////////////////////////////////////////////
// t@CSPARTǍ
////////////////////////////////////////////////////////////////
bool cP6PART::Readf( FILE *fp )
{
	PRINTD( P6T2_LOG, "[cP6PART][Readf]\n" );
	
	// t@C|C^͗L?
	if( !fp ) return false;
	
	// ʎq "TI"(0x4954) ?
	while( ( FGETWORD( fp ) ) == 0x4954 ){
		
		BYTE NextID = fgetc( fp );	// IDԍ
		
		// DATAubN,IDقȂȂVKPARTǉ
		if( LastPart()->Data && ( LastPart()->ID != NextID ) ){
			New();
			LastPart()->ID = NextID;
		}
		
		// DATAubN?
		if( LastPart()->Data )
			// pPARTȂf[^ƃ{[[gǂݔ΂
			fseek( fp, sizeof(Name)+sizeof(Baud)-1, SEEK_CUR );
		else{
			fread( LastPart()->Name, sizeof(BYTE), sizeof(Name)-1, fp );	// f[^
			LastPart()->Baud = FGETWORD( fp );								// {[[g
		}
		// DATAubNǉ
		cP6DATA *NewBlk = LastPart()->NewBlock();
		
		WORD stime = FGETWORD( fp );	// ̎(ms)
		WORD ptime = FGETWORD( fp );	// ҁ[̎(ms)
		DWORD doff = FGETDWORD( fp );	// x^C[W擪̃ItZbg
		DWORD dnum = FGETDWORD( fp );	// f[^TCY
		
		// f[^Ǎ
		DWORD fps = ftell( fp );
		fseek( fp, doff, SEEK_SET );
		NewBlk->SetData( fp, dnum );
		NewBlk->SetPeriod( stime, ptime );
		fseek( fp, fps, SEEK_SET );	// t@C|C^PARTɖ߂
	}
	
	Renumber();
	
	return true;
}


////////////////////////////////////////////////////////////////
// t@Cɏ(f[^)
////////////////////////////////////////////////////////////////
int cP6PART::Writefd( FILE *fp )
{
	cP6DATA *data = Data;
	int size = 0;
	
	do{
		size += data->Writefd( fp );
		data = data->Next();
	}while( data );
	
	return size;
}


////////////////////////////////////////////////////////////////
// t@Cɏ(tb^)
////////////////////////////////////////////////////////////////
bool cP6PART::Writeff( FILE *fp )
{
// [DATAubN]
//  header (2byte) : "TI"
//  id     (1byte) : IDԍ(DATAubN֘At)
//  name  (16byte) : f[^(15+'00H')
//  baud   (2byte) : {[[g(600/1200)
//  stime  (2byte) : ̎(ms)
//  ptime  (2byte) : ҁ[̎(ms)
//  offset (4byte) : x^C[W擪̃ItZbg
//  size   (4byte) : f[^TCY
	
	cP6DATA *data = Data;
	
	do{
		fputc( 'T', fp );
		fputc( 'I', fp );
		fputc( ID, fp );
		fwrite( Name, sizeof(BYTE), 16, fp );
		FPUTWORD( Baud, fp )
		
		P6TBLKINFO info;
		data->GetInfo( &info );
		FPUTWORD( info.STime, fp );
		FPUTWORD( info.PTime, fp );
		FPUTDWORD( info.Offset, fp );
		FPUTDWORD( info.DNum, fp );
		
		data = data->Next();
	}while( data );
	
	return true;
}



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
cP6T::cP6T( void ) : Version(0), Start(false), BASIC(1), Page(1), ASKey(0),
						EHead(0), ask(NULL), exh(NULL), Part(NULL), Boost(1),
						rpart(NULL), rdata(NULL), rpt(0), swait(0), pwait(0)
{
	INITARRAY( Name, 0 );
}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
cP6T::~cP6T( void )
{
	if( Part ) delete Part;
	if( ask ) delete [] ask;
	if( exh ) delete [] exh;
}


////////////////////////////////////////////////////////////////
// SDATAubN擾
////////////////////////////////////////////////////////////////
int cP6T::GetBlocks( void )
{
	int num = 0;
	cP6PART *part = Part;
	
	do{
		num += part->GetBlocks();
		part = part->Next();
	}while( part );
	
	return num;
}


////////////////////////////////////////////////////////////////
// SPART擾
////////////////////////////////////////////////////////////////
int cP6T::GetParts( void )
{
	int num = 0;
	cP6PART *part = Part;
	
	do{
		num ++;
		part = part->Next();
	}while( part );
	
	return num;
}


////////////////////////////////////////////////////////////////
// x^C[WTCY擾
////////////////////////////////////////////////////////////////
int cP6T::GetSize( void )
{
	int size = 0;
	cP6PART *part = Part;
	
	while( part ){
		size += part->GetSize();
		part  = part->Next();
	}
	
	return size;
}


////////////////////////////////////////////////////////////////
// f[^ݒ
////////////////////////////////////////////////////////////////
int cP6T::SetName( const char *name )
{
//	ZeroMemory( Name, sizeof(Name) );
//	strncpy( Name, name, sizeof(Name)-1 );
	ZeroMemory( Name, 16 );
	strncpy( Name, name, 16 );
	
	return strlen( Name );
}


////////////////////////////////////////////////////////////////
// 1Ǎ
////////////////////////////////////////////////////////////////
BYTE cP6T::ReadOne( void )
{
	P6TBLKINFO binfo;
	
	BYTE data = rdata->Read( rpt++ );
	
	PRINTD( P6T2_LOG, "[cP6T][ReadOne] -> %02X\n", data );
	
	// ubN擾
	rdata->GetInfo( &binfo );
	if( rpt >= binfo.DNum ){	// f[^Ō?
		rpt = 0;
		if( !( rdata = rdata->Next() ) ){	// DATAubNŌ?
			if( !( rpart = rpart->Next() ) ){	// PARTŌ?
				rpart = Part;
			}
			rdata = rpart->FirstData();
		}
		// ̃ubN擾
		rdata->GetInfo( &binfo );
		swait = ( binfo.STime * DEFAULT_CMT_HZ * Boost )/1000;
		pwait = ( binfo.PTime * DEFAULT_CMT_HZ * Boost )/1000;
	}
	
	return data;
}


////////////////////////////////////////////////////////////////
// ҂?
////////////////////////////////////////////////////////////////
bool cP6T::IsSWaiting( void )
{
	if( !swait ) return false;
	swait--;
	return true;
}


////////////////////////////////////////////////////////////////
// ҁ[҂?
////////////////////////////////////////////////////////////////
bool cP6T::IsPWaiting( void )
{
	if( !pwait ) return false;
	pwait--;
	return true;
}


////////////////////////////////////////////////////////////////
// Zbg(Ǎ݊֌W[N)
////////////////////////////////////////////////////////////////
void cP6T::Reset( void )
{
	PRINTD( P6T2_LOG, "[cP6T][Reset]\n" );
	
	// Ǎ݊֌W[N
	rpart = Part;				// ݂̓ǍPART
	rdata = Part->FirstData();	// ݂̓ǍDATAubN
	rpt   = 0;					// ݂̓Ǎ݃|C^
	
	P6TBLKINFO binfo;
	
	rdata->GetInfo( &binfo );
	swait = ( binfo.STime * DEFAULT_CMT_HZ * Boost )/1000;	// ̑҂
	pwait = ( binfo.PTime * DEFAULT_CMT_HZ * Boost )/1000;	// ҁ[̑҂
}


////////////////////////////////////////////////////////////////
// {[[g{ݒ
////////////////////////////////////////////////////////////////
void cP6T::SetBoost( int boost )
{
	Boost = boost;
}


////////////////////////////////////////////////////////////////
// t@CǍ
////////////////////////////////////////////////////////////////
bool cP6T::Readf( char *filename )
{
	PRINTD( P6T2_LOG, "[cP6T][Readf] [%s]\n", filename );
	
	if( !ReadP6T( filename ) ){		// P6TǍ
		// sx^Ƃ݂ȂP6TɕϊēǍ
		if( !ConvP6T( filename ) ) return false;
	}
	
	Reset();	// Zbg(Ǎ݊֌W[N)
	
	return true;
}


////////////////////////////////////////////////////////////////
// t@Cɏ
////////////////////////////////////////////////////////////////
bool cP6T::Writef( char *filename )
{
	PRINTD( P6T2_LOG, "[cP6T][Writef] [%s]\n", filename );
	
	FILE *fp;
	
	fp = FOPENEN( filename, "wb" );
	if( !fp ) return false;
	
	// f[^&x^C[WTCY擾
	DWORD BetaSize = 0;
	cP6PART *part = Part;
	do{
		BetaSize += part->Writefd( fp );
		part = part->Next();
	}while( part );
	

// [tb^]
//  header (2byte) : "P6"
//  ver    (1byte) : o[W
//  dbnum  (1byte) : ܂܂DATAubN(255܂)
//  start  (1byte) : I[gX^[gtO(0: 1:L)
//  basic  (1byte) : BASIC[h(PC-6001̏ꍇ͖Ӗ)
//  page   (1byte) : y[W
//  askey  (2byte) : I[gX^[gR}h
//  ...R}hꍇ͂̌askey
//  exhead (2byte) : gTCY(64KB܂)
//  ...g񂪂ꍇ͂̌exhead

	// P6T tb^
	fputc( 'P', fp );
	fputc( '6', fp );
	fputc( Version, fp );
	fputc( GetBlocks(), fp );
	fputc( Start ? 1 : 0, fp );
	fputc( BASIC, fp );
	fputc( Page, fp );
	FPUTWORD( ASKey, fp );
	if( ASKey ) fwrite( ask, sizeof(char), ASKey, fp );
	FPUTWORD( EHead, fp );
	if( EHead ) fwrite( exh, sizeof(char), EHead, fp );
	
	// SDATAubNtb^
	part = Part;
	do{
		part->Writeff( fp );
		part = part->Next();
	}while( part );
	
	// x^C[WTCY
	FPUTDWORD( BetaSize, fp );
	
	return true;
}


////////////////////////////////////////////////////////////////
// P6TǍ
////////////////////////////////////////////////////////////////
bool cP6T::ReadP6T( const char *filename )
{
	PRINTD( P6T2_LOG, "[cP6T][ReadP6T] [%s]\n", filename );
	
	FILE *fp;
	
	fp = FOPENEN( filename, "rb" );
	if( !fp ) return false;
	
	// x^C[WTCY擾
	fseek( fp, -4, SEEK_END );
	DWORD BetaSize = FGETDWORD( fp );
	// x^C[WTCYt@CTCY𒴂ĂG[
	if( BetaSize > (DWORD)ftell( fp ) ){ fclose( fp ); return false; }
	
	// tb^̐擪Ɉړ
	fseek( fp, BetaSize, SEEK_SET );
	
	// ʎq "P6"(0x3650) łȂ΃G[
	WORD Header = FGETWORD( fp );
	if( Header != 0x3650 ){ fclose( fp ); return false; }
	
	Version = fgetc( fp );					// o[W
	fgetc( fp );							// ܂܂DATAubN
	Start   = fgetc( fp ) ? true : false;	// I[gX^[gtO
	BASIC   = fgetc( fp );					// BASIC[h
	Page    = fgetc( fp );					// y[W
	
	// I[gX^[gR}h
	ASKey = FGETWORD( fp );
	if( ASKey ){
		ask = new char[ASKey];
		fread( ask, sizeof(char), ASKey, fp );
	}
	
	// g
	EHead = FGETWORD( fp );
	if( EHead ){
		exh = new char[EHead];
		fread( exh, sizeof(char), EHead, fp );
	}
	
	// PARTǍ
	Part = new cP6PART;
	if( !Part->Readf( fp ) ){ fclose( fp ); return false; }
	
	fclose( fp );
	
	return true;
}


////////////////////////////////////////////////////////////////
// x^P6TɕϊēǍ
////////////////////////////////////////////////////////////////
bool cP6T::ConvP6T( const char *filename )
{
	PRINTD( P6T2_LOG, "[cP6T][ConvP6T] [%s]\n", filename );
	
	FILE *fp;
	
	fp = FOPENEN( filename, "rb" );
	if( !fp ) return false;
	
	// x^C[WTCY擾
	fseek( fp, 0, SEEK_END );
	DWORD BetaSize = ftell( fp );
	fseek( fp, 0, SEEK_SET );
	
	// P6Tݒ
	SetName( OSD_GetFileNamePart( filename ) );	// f[^(16+'00H')̓t@C
	Version = 2;								// o[W(Ƃ肠2)
	Start   = false;							// I[gX^[gtO()
	BASIC   = 1;								// BASIC[h(PC-6001̏ꍇ͖Ӗ)(Ƃ肠1Ӗ)
	Page    = 1;								// y[W(Ƃ肠1Ӗ)
	ASKey   = 0;								// I[gX^[gR}hTCY(0:)
	EHead   = 0;								// gTCY(0:Ȃ)
	ask     = NULL;								// I[gX^[gR}hi[̈ւ̃|C^
	exh     = NULL;								// gi[̈ւ̃|C^
	
	// PART쐬
	Part = new cP6PART;
	Part->SetName( "BetaImage" );	// f[^(16+'00H')͂Ƃ肠"BetaImage"
	Part->SetBaud( 1200 );			// {[[g(1200)
	
	// DATAubN쐬
	Part->NewBlock();
	Part->FirstData()->SetData( fp, BetaSize );	// f[^Zbg
	Part->FirstData()->SetPeriod( 3400, 6800 );	// (3400ms),ҁ[(6800ms)ԃZbg
	
	fclose( fp );
	
	return true;
}


////////////////////////////////////////////////////////////////
// TAPEJE^擾
////////////////////////////////////////////////////////////////
int cP6T::GetCount( void )
{
	if( !rdata ) return 0;
	
	P6TBLKINFO info;
	rdata->GetInfo( &info );
	
	return info.Offset + rpt;
}


////////////////////////////////////////////////////////////////
// TAPEJE^ݒ
////////////////////////////////////////////////////////////////
void cP6T::SetCount( int cnt )
{
	if( GetSize() <= cnt ) return;
	
	int off = 0;
	
	// ܂PART`FbN
	rpart = Part;
	while( off+rpart->GetSize() <= cnt ){
		off   += rpart->GetSize();
		rpart  = rpart->Next();
	}
	// DATA`FbN
	rdata = rpart->FirstData();
	while( off+rdata->GetSize() <= cnt ){
		off   += rdata->GetSize();
		rdata  = rdata->Next();
	}
	
	P6TBLKINFO info;
	rdata->GetInfo( &info );
	
	rpt = cnt - info.Offset;
}


////////////////////////////////////////////////////////////////
// ǂłSAVE
////////////////////////////////////////////////////////////////
bool cP6T::DokoSave( cIni *Ini )
{
	if( !Ini ) return false;
	
	Ini->PutEntry( "P6T", NULL, "Counter",	"%d",	GetCount() );
	Ini->PutEntry( "P6T", NULL, "swait",	"%d",	swait );
	Ini->PutEntry( "P6T", NULL, "pwait",	"%d",	pwait );
	
	return true;
}


////////////////////////////////////////////////////////////////
// ǂłLOAD
////////////////////////////////////////////////////////////////
bool cP6T::DokoLoad( cIni *Ini )
{
	int st;
	
	if( !Ini ) return false;
	
	Ini->GetInt( "P6T",	"Counter",	&st,	0 );
	SetCount( st );
	Ini->GetInt( "P6T",	"swait",	&swait,	swait );
	Ini->GetInt( "P6T",	"pwait",	&pwait,	pwait );
	
	return true;
}
