#include <stdlib.h>
#include <new>

#include "log.h"
#include "replay.h"
#include "common.h"
#include "error.h"


////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
REPLAY::REPLAY( void ) : Ini(NULL), RepST(REP_IDLE), Matrix(NULL),
							MSize(0), RepFrm(0), EndFrm(0) {}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
REPLAY::~REPLAY( void )
{
	switch( RepST ){
	case REP_RECORD:	StopRecord(); break;
	case REP_REPLAY:	StopReplay(); break;
	}
	if( Matrix ) delete [] Matrix;
	if( Ini ) delete Ini;
}


////////////////////////////////////////////////////////////////
// 
//
// :	msize	}gNXTCY
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool REPLAY::Init( int msize )
{
	PRINTD( GRP_LOG, "[REPLAY][Init]\n" );
	
	if( Ini ) delete Ini;
	Ini = NULL;
	
	if( Matrix ) delete [] Matrix;
	Matrix = new BYTE[msize];
	if( !Matrix ) return false;
	
	RepST   = REP_IDLE;
	MSize   = msize;
	RepFrm  = 0;
	EndFrm  = 0;
	
	return true;
}


////////////////////////////////////////////////////////////////
// Xe[^X擾
//
// :	Ȃ
// Ԓl:	int		Xe[^X
////////////////////////////////////////////////////////////////
int REPLAY::GetStatus( void )
{
	return RepST;
}


////////////////////////////////////////////////////////////////
// vCL^Jn
//
// :	filename	o̓t@C
// Ԓl:	bool		true: false:s
////////////////////////////////////////////////////////////////
bool REPLAY::StartRecord( char *filename )
{
	// Ƃ肠G[ݒ
	Error::SetError( Error::ReplayPlayError );
	try{
		if( RepST != REP_IDLE ) throw Error::ReplayRecError;
		
		Ini = new cIni();
		if( !Ini->Init( filename ) ) throw Error::ReplayRecError;
	}
	catch( std::bad_alloc ){	// new Ɏsꍇ
		Error::SetError( Error::MemAllocFailed );
		return false;
	}
	catch( Error::Errno i ){	// O
		Error::SetError( i );
		if( Ini ) delete Ini;
		Ini = NULL;
		return false;
	}
	
	memset( Matrix, 0xff, MSize );		// L[}gNXobt@NA
	
	RepFrm = 0;
	RepST  = REP_RECORD;
	
	// ̂ŃG[Ȃ
	Error::SetError( Error::NoError );
	
	return true;
}


////////////////////////////////////////////////////////////////
// vCL^~
//
// :	Ȃ
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void REPLAY::StopRecord( void )
{
	if( RepST != REP_RECORD ) return;
	
	if( Ini ){
		Ini->PutEntry( "REPLAY", NULL, "EndFrm", "0x%08lX", RepFrm );
		
		Ini->Write();
		
		delete Ini;
		Ini = NULL;
	}
	
	RepST = REP_IDLE;
}


////////////////////////////////////////////////////////////////
// vC1t[o
//
// :	mt		L[}gNX|C^
// 			chg		L[}gNXω true: false:Ȃ
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool REPLAY::ReplayWriteFrame( BYTE *mt, bool chg )
{
	char stren[16],strva[256];
	
	if( ( RepST != REP_RECORD ) || !mt || !Ini ) return false;
	
	// }gNXɕω珑o
	if( chg ){
		sprintf( stren, "%08lX ", RepFrm );
		for( int i=0; i<MSize; i++ )
			sprintf( strva+i*2, "%02X", mt[i] );
		Ini->PutEntry( "REPLAY", NULL, stren, "%s", strva );
	}
	
	RepFrm++;
	
	return true;
}


////////////////////////////////////////////////////////////////
// vCĐJn
//
// :	filename	̓t@C
// Ԓl:	bool		true: false:s
////////////////////////////////////////////////////////////////
bool REPLAY::StartReplay( char *filename )
{
	// Ƃ肠G[ݒ
	Error::SetError( Error::ReplayPlayError );
	try{
		int st;	// zgDWORD
		
		if( RepST != REP_IDLE ) throw Error::ReplayPlayError;
		
		Ini = new cIni();
		if( !Ini->Init( filename ) ) throw Error::ReplayPlayError;
		
		if( !Ini->GetInt( "REPLAY", "EndFrm", &st, EndFrm ) ) throw Error::NoReplayData;
		else                                                  EndFrm = st;
	}
	catch( std::bad_alloc ){	// new Ɏsꍇ
		Error::SetError( Error::MemAllocFailed );
		return false;
	}
	catch( Error::Errno i ){	// O
		Error::SetError( i );
		if( Ini ) delete Ini;
		Ini = NULL;
		return false;
	}
	
	memset( Matrix, 0xff, MSize );	// L[}gNXobt@NA
	
	RepFrm = 0;
	RepST  = REP_REPLAY;
	
	// ̂ŃG[Ȃ
	Error::SetError( Error::NoError );
	
	return true;
}


////////////////////////////////////////////////////////////////
// vCĐ~
//
// :	Ȃ
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void REPLAY::StopReplay( void )
{
	if( RepST != REP_REPLAY ) return;
	
	if( Ini ){
		delete Ini;
		Ini = NULL;
	}
	
	RepST = REP_IDLE;
}


////////////////////////////////////////////////////////////////
// vC1t[Ǎ
//
// :	mt		L[}gNX|C^
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool REPLAY::ReplayReadFrame( BYTE *mt )
{
	char stren[16],strva[256];
	
	if( ( RepST != REP_REPLAY ) || !mt || !Ini ) return false;
	
	sprintf( stren, "%08lX", RepFrm );
	if( Ini->GetString( "REPLAY", stren, strva, "" ) ){
		int stl = strlen( strva ) / 2;
		for( int i=0; i<stl; i++ ){
			char dt[3] = "FF";
			strncpy( dt, &strva[i*2], 2 );
			Matrix[i] = strtol( dt, NULL, 16 );
		}
		memcpy( mt, Matrix, MSize );
	}
	
	if( ++RepFrm >= EndFrm ){
		// f[^I[ɒB烊vCI
		StopReplay();
	}
	
	return true;
}
