#include "p6el.h"

#include "log.h"
#include "status.h"
#include "cpus.h"
#include "disk.h"
#include "keyboard.h"
#include "replay.h"
#include "tape.h"


//------------------------------------------------------
//  Xe[^Xo[NX
//------------------------------------------------------
////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
cWndStat::cWndStat( VM6 *pvm ) : vm(pvm),TapeIsAutoStart(false),
	TapeIsOpen(false), TapeSize(0), TapeCount(0), DrvNum(0),
	KeyIndicator(0), ReplayStatus(0)
{
	INITARRAY( TapeName, '\0' );
	INITARRAY( DiskName[0], '\0' );
	INITARRAY( DiskName[1], '\0' );
	INITARRAY( DiskIsSystem, false );
	INITARRAY( DiskIsProtect, false );
	INITARRAY( DiskIsAccess, false );
}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
cWndStat::~cWndStat( void ){}


////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////
bool cWndStat::Init( int w, int drv )
{
	PRINTD( WIN_LOG, "[WndStat][Init]\n" );
	
	if( drv >= 0 ) DrvNum = drv;
	ZCons::SetColor( FC_WHITE, FC_GRAY );
	
	return ZCons::InitRes( w < 0 ? VSurface::Width() : w , JFont::FontHeight()*(DrvNum+1) + 4, NULL, FC_WHITE, FC_GRAY );
}


////////////////////////////////////////////////////////////////
// EBhEXV
////////////////////////////////////////////////////////////////
void cWndStat::Update( void )
{
	PRINTD( WIN_LOG, "[WndStat][Update]\n" );
	
	const BYTE Kana[]  = { 0x96, 0xe5, 0 };	// 
	const BYTE KKana[] = { 0xb6, 0xc5, 0 };	// Ji
	
	
	TapeIsOpen   = ( vm->cpus->GetCmtStatus() == LOADOPEN );
	TapeCount    = vm->cmtl->GetCount();
	KeyIndicator = vm->key->GetKeyIndicator();
	
	ZCons::Cls();
	ZCons::SetColor( FC_WHITE );
	
	// TAPE
	ZCons::Locate( 0, 0 );
	ZCons::Printf( "[TAPE]" );
	if( strlen( TapeName ) ){
		ZCons::SetColor( TapeIsAutoStart ? FC_YELLOW : FC_WHITE );
		ZCons::Printf( " %-16s", TapeName );
		ZCons::SetColor( FC_WHITE );
		ZCons::Locate( ZCons::GetXline()-19, 0 );
		if( TapeIsOpen ) ZCons::SetColor( FC_WHITE, FC_MAGENTA );
		ZCons::Printf( "[%05d/%05d]", TapeCount, TapeSize );
		ZCons::SetColor( FC_WHITE, FC_GRAY );
	}
	
	// DISK
	if( DrvNum > 0 ){
		if( vm->disk->InAccess( 0 ) ) ZCons::SetColor( FC_WHITE, FC_RED );
		else						  ZCons::SetColor( FC_WHITE, FC_GRAY );
		ZCons::Locate( 0, 1 );
		ZCons::Printf( "[DRV1]" );
		if( strlen( DiskName[0] ) ){
			ZCons::SetColor( DiskIsSystem[0] ? FC_YELLOW : FC_WHITE, DiskIsProtect[0] ? FC_DRED : FC_GRAY );
			ZCons::Printf( " %-16s", DiskName[0] );
		}
	}
	if( DrvNum > 1 ){
		if( vm->disk->InAccess( 1 ) ) ZCons::SetColor( FC_WHITE, FC_RED );
		else						  ZCons::SetColor( FC_WHITE, FC_GRAY );
		ZCons::Locate( 0, 2 );
		ZCons::Printf( "[DRV2]" );
		if( strlen( DiskName[1] ) ){
			ZCons::SetColor( DiskIsSystem[1] ? FC_YELLOW : FC_WHITE, DiskIsProtect[1] ? FC_DRED : FC_GRAY );
			ZCons::Printf( " %-16s", DiskName[1] );
		}
		// ANZXv
	}
	ZCons::SetColor( FC_WHITE, FC_GRAY );
	
	// ȃL[
	ZCons::Locate( -5, 0 );
	switch( KeyIndicator&3 ){
	case KI_KANA:	// 
		ZCons::PutCharH( Kana[0] );
		ZCons::PutCharH( Kana[1] );
		break;
	case KI_KKANA:	// Ji
		ZCons::PutCharH( KKana[0] );
		ZCons::PutCharH( KKana[1] );
	}
	
	// CAPSL[
	if( KeyIndicator&4 ) ZCons::Printfr( "ABC" );	// ABC
	else                 ZCons::Printfr( "abc" );	// abc
	
	// vCXe[^X
	ZCons::Locate( -2, 0 );
	switch( ReplayStatus ){
	case REP_RECORD:	// L^
		ZCons::SetColor( FC_RED );
		ZCons::Print( "" );
		break;
	case REP_REPLAY:	// Đ
		ZCons::SetColor( FC_GREEN );
		ZCons::Print( "" );
		break;
	}
}



////////////////////////////////////////////////////////////////
// TAPE }Eg
//
// :	filename	t@C
//			astart		I[gX^[g?
//			size		TCY
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void cWndStat::TapeMount( char *filename, bool astart, int size )
{
	strcpy( TapeName, filename );
	TapeIsAutoStart = astart;
	TapeSize        = size;
}


////////////////////////////////////////////////////////////////
// TAPE A}Eg
//
// :	Ȃ
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void cWndStat::TapeUnmount( void )
{
	*TapeName = '\0';
	TapeIsAutoStart = false;
	TapeIsOpen      = false;
	TapeSize        = 0;
	TapeCount       = 0;
}


////////////////////////////////////////////////////////////////
// DISK }Eg
//
// :	drv			hCuԍ
//			filename	t@C
//			sys			VXefBXN?
//			prot		veNg?
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void cWndStat::DiskMount( int drv, char *filename, bool sys, bool prot )
{
	strcpy( DiskName[drv], filename );
	DiskIsSystem[drv]  = sys;
	DiskIsProtect[drv] = prot;
}


////////////////////////////////////////////////////////////////
// DISK A}Eg
//
// :	drv			hCuԍ
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void cWndStat::DiskUnmount( int drv )
{
	*DiskName[drv] = '\0';
	DiskIsSystem[drv]  = false;
	DiskIsProtect[drv] = false;
	DiskIsAccess[drv]  = false;
}


////////////////////////////////////////////////////////////////
// vCXe[^XZbg
//
// :	stat		vCXe[^X
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void cWndStat::SetReplayStatus( int stat )
{
	ReplayStatus = stat;
}
