#include <new>
#include <limits.h>

#include "common.h"
#include "vsurface.h"

#define	RMASK32	0x00ff0000
#define	GMASK32	0x0000ff00
#define	BMASK32	0x000000ff

#define	RMASK16	0x0000f800
#define	GMASK16	0x000007e0
#define	BMASK16	0x0000001f


////////////////////////////////////////////////////////////////
// RXgN^
////////////////////////////////////////////////////////////////
VSurface::VSurface( void ) : w(0), h(0), bpp(0), pitch(0), pixels(NULL), palette(NULL),
	rmask(0), gmask(0), bmask(0), rshift(0), gshift(0),bshift(0),
	rloss(0), gloss(0), bloss(0) {}


////////////////////////////////////////////////////////////////
// fXgN^
////////////////////////////////////////////////////////////////
VSurface::~VSurface( void )
{
	if( pixels ) delete [] (BYTE *)pixels;
	
	if( palette ){
		if( palette->colors ) delete [] palette->colors;
		delete palette;
	}
}



////////////////////////////////////////////////////////////////
// (T[tFXTCYw)
//
// :	ww,hh	,
//			bpp		1sNZbit
// Ԓl:	bool	true: false:s
////////////////////////////////////////////////////////////////
bool VSurface::InitSurface( int ww, int hh, int bp )
{
	// TCY`FbN
	if( ww <= 0 || ww > 0xffff || hh <= 0 || hh > 0xffff ) return false;
	
	if( bp != 8 && bp != 16 && bp != 24 ) return false;
	
	// 쐬ς݂Ȃ炢J
	if( pixels ) delete [] (BYTE *)pixels;
	if( palette ){
		if( palette->colors ) delete [] palette->colors;
		delete palette;
		palette = NULL;
	}
	
	// m
	pitch = (ww*(bp/8)+3)&0xfffffffc;
	pixels = new BYTE[pitch*hh];
	if( !pixels ){
		w = h = bpp = pitch = rect.x = rect.y = rect.w = rect.h = 0;
		return false;
	}
	
	// pbg쐬(8bpp̂)
	if( bp == 8 ){
		palette = new VPalette;
		palette->ncols  = 256;
		palette->colors = new COLOR24[256];
		
		for( int i=0; i<palette->ncols; i++ ){
			palette->colors[i].r        = 0;
			palette->colors[i].g        = 0;
			palette->colors[i].b        = 0;
			palette->colors[i].reserved = 0;
		}
	}
	
	w = ww;
	h = hh;
	bpp = bp;
	rect.x = rect.y = 0;
	rect.w = w;
	rect.h = h;
	
	switch( bpp ){
	case 16: SetMask( RMASK16, GMASK16, BMASK16 ); break;
	case 24: SetMask( RMASK32, GMASK32, BMASK32 ); break;
	}
	
	return true;
}


////////////////////////////////////////////////////////////////
// `̈ݒ
// :	xx,yy	W
//			ww,hh	,
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void VSurface::SetRect( int xx, int yy, int ww, int hh )
{
	VRect r1,r2;
	
	r1.x = 0;
	r1.y = 0;
	r1.w = w;
	r1.h = h;
	
	r2.x = xx;
	r2.y = xx;
	r2.w = ww > 0 ? ww : 0;
	r2.h = hh > 0 ? hh : 0;
	
	RectAdd( &rect, &r1, &r2 );
}


////////////////////////////////////////////////////////////////
// `̈擾
// :	Ȃ
// Ԓl:	VRect *	`̈ւ̃|C^
////////////////////////////////////////////////////////////////
VRect *VSurface::GetRect( void )
{
	return &rect;
}


////////////////////////////////////////////////////////////////
// pbgݒ
// :	colors	pbgf[^ւ̃|C^
//			ncols	ݒ肷F
// Ԓl:	int		ݒ肳ꂽF
////////////////////////////////////////////////////////////////
int VSurface::SetPalette( COLOR24 *cols, int ncols )
{
	if( !cols || (ncols < 1) || (bpp != 8) ) return 0;
	
	palette->ncols  = ncols;
	for( int i=0; i<palette->ncols; i++ ){
		palette->colors[i].r = cols[i].r;
		palette->colors[i].g = cols[i].g;
		palette->colors[i].b = cols[i].b;
	}
	
	return palette->ncols;
}


////////////////////////////////////////////////////////////////
// pbg擾
// :	Ȃ
// Ԓl:	VPalette *	pbgւ̃|C^
////////////////////////////////////////////////////////////////
VPalette *VSurface::GetPalette( void )
{
	return palette;
}


////////////////////////////////////////////////////////////////
// J[}XNݒ
// :	rm			ԃ}XN
//			gm			΃}XN
//			bm			}XN
// Ԓl:	VPalette *	pbgւ̃|C^
////////////////////////////////////////////////////////////////
void VSurface::SetMask( DWORD rm, DWORD gm, DWORD bm )
{
	rmask = rm;
	gmask = gm;
	bmask = bm;
	rshift = gshift = bshift = 0;
	rloss  = gloss  = bloss  = 0;
	
	switch( bpp ){
	case 16:
		while( !(rmask&1) && rshift<16 ){ rshift++; rmask >>= 1; }
		while( !(gmask&1) && gshift<16 ){ gshift++; gmask >>= 1; }
		while( !(bmask&1) && bshift<16 ){ bshift++; bmask >>= 1; }
		while( !(rmask&0x80) && rloss<8 ){ rloss++; rmask <<= 1; }
		while( !(gmask&0x80) && gloss<8 ){ gloss++; gmask <<= 1; }
		while( !(bmask&0x80) && bloss<8 ){ bloss++; bmask <<= 1; }
		break;
		
	case 24:
		while( !(rmask&1) && rshift<24 ){ rshift++; rmask >>= 1; }
		while( !(gmask&1) && gshift<24 ){ gshift++; gmask >>= 1; }
		while( !(bmask&1) && bshift<24 ){ bshift++; bmask >>= 1; }
		while( !(rmask&0x80) && rloss<8 ){ rloss++; rmask <<= 1; }
		while( !(gmask&0x80) && gloss<8 ){ gloss++; gmask <<= 1; }
		while( !(bmask&0x80) && bloss<8 ){ bloss++; bmask <<= 1; }
	}
	
	rmask = rm;
	gmask = gm;
	bmask = bm;
}


////////////////////////////////////////////////////////////////
// RGB}XN擾
// :	Ȃ
// Ԓl:	DWORD		}XN
////////////////////////////////////////////////////////////////
DWORD VSurface::Rmask( void ){ return rmask; }
DWORD VSurface::Gmask( void ){ return gmask; }
DWORD VSurface::Bmask( void ){ return bmask; }


////////////////////////////////////////////////////////////////
// RGBEVtg擾
// :	Ȃ
// Ԓl:	int		EVtg
////////////////////////////////////////////////////////////////
int VSurface::Rshift( void ){ return rshift; }
int VSurface::Gshift( void ){ return gshift; }
int VSurface::Bshift( void ){ return bshift; }


////////////////////////////////////////////////////////////////
// RGBVtg擾
// :	Ȃ
// Ԓl:	int		Vtg
////////////////////////////////////////////////////////////////
int VSurface::Rloss( void ){ return rloss; }
int VSurface::Gloss( void ){ return gloss; }
int VSurface::Bloss( void ){ return bloss; }


////////////////////////////////////////////////////////////////
// 擾
// :	Ȃ
// Ԓl:	int		
////////////////////////////////////////////////////////////////
int VSurface::Width( void )
{
	return w;
}


////////////////////////////////////////////////////////////////
// 擾
// :	Ȃ
// Ԓl:	int		
////////////////////////////////////////////////////////////////
int VSurface::Height( void )
{
	return h;
}


////////////////////////////////////////////////////////////////
// 1sNZbyte擾
// :	Ȃ
// Ԓl:	int		1sNZbyte
////////////////////////////////////////////////////////////////
int VSurface::Bpp( void )
{
	return bpp;
}


////////////////////////////////////////////////////////////////
// 1Cbyte擾
// :	Ȃ
// Ԓl:	int		1Cbyte
////////////////////////////////////////////////////////////////
int VSurface::Pitch( void )
{
	return pitch;
}


////////////////////////////////////////////////////////////////
// sNZf[^ւ̃|C^擾
// :	Ȃ
// Ԓl:	BYTE *	sNZf[^ւ̃|C^
////////////////////////////////////////////////////////////////
void *VSurface::GetPixels( void )
{
	return pixels;
}


////////////////////////////////////////////////////////////////
// _`
//
// :	x,y				W
//			col				J[R[h
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void VSurface::PSet( int x, int y, DWORD col )
{
	BYTE *p;
	
	if( pixels && x >= rect.x && x < rect.x + rect.w && y >= rect.y && y < rect.y + rect.h ){
		p = (BYTE *)pixels + y * pitch + x * bpp / 8;
		switch( bpp ){
		case 8:
			*p   =  col     &0xff;
			break;
			
		case 16:
			*p++ =  col     &0xff;
			*p   = (col>> 8)&0xff;
			break;
			
		case 24:
			*p++ =  col     &0xff;
			*p++ = (col>> 8)&0xff;
			*p   = (col>>16)&0xff;
			break;
		default:
			break;
		}
	}
}


////////////////////////////////////////////////////////////////
// F擾
//
// :	x,y				W
// Ԓl:	BYTE			J[R[h
////////////////////////////////////////////////////////////////
DWORD VSurface::PGet( int x, int y )
{
	BYTE *p;
	DWORD res = 0;
	
	if( pixels && x >= rect.x && x < rect.x + rect.w && y >= rect.y && y < rect.y + rect.h ){
		p = (BYTE *)pixels + y * pitch + x * bpp / 8;
		switch( bpp ){
		case 8:
			res  = (DWORD)(*p);
			break;
			
		case 16:
			res  = (DWORD)(*p++);
			res |= (DWORD)(*p  )<< 8;
			break;
			
		case 24:
			res  = (DWORD)(*p++);
			res |= (DWORD)(*p++)<< 8;
			res |= (DWORD)(*p  )<<16;
			break;
		default:
			break;
		}
	}
	return res;
}


////////////////////////////////////////////////////////////////
// `̈hԂ
// :	col		F
//			rc		hԂ͈(NULLȂl)
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void VSurface::Fill( DWORD col, VRect *rc )
{
	VRect rr;
	
	if( !pixels ) return;
	
	if( rc ){
		RectAdd( &rr, &rect, rc );
	}else{
		rr.x = rect.x;
		rr.y = rect.y;
		rr.w = rect.w;
		rr.h = rect.h;
	}
	
	if( rr.w && rr.h ){
		BYTE *p = (BYTE *)pixels + rr.y * pitch + rr.x * bpp / 8;
		for( int i=0; i < rr.h; i++ ){
			switch( bpp ){
			case 8:
				memset( p, col, rr.w );
				break;
				
			case 16:
				for( int j=0; j < rr.w; j++ ){
					*p     =  col     &0xff;
					*(p+1) = (col>> 8)&0xff;
				}
				break;
				
			case 24:
				for( int j=0; j < rr.w; j++ ){
					*p     =  col     &0xff;
					*(p+1) = (col>> 8)&0xff;
					*(p+2) = (col>>16)&0xff;
				}
				break;
			default:
				break;
			}
			p += pitch;
		}
	}
}


////////////////////////////////////////////////////////////////
// ]
//
// :	srect			]͈
//			dst				]T[tFXւ̃|C^
//			drect			]͈(x,ŷݗL)
// Ԓl:	Ȃ
////////////////////////////////////////////////////////////////
void VSurface::Blit( VRect *srect, VSurface *dst, VRect *drect )
{
	VRect src1,drc1,src2,drc2;
	
	if( !dst ) return;
	
	// ]͈͐ݒ
	src1.x = srect ? srect->x : 0;
	src1.y = srect ? srect->y : 0;
	src1.w = srect ? srect->w : w;
	src1.h = srect ? srect->h : h;
	RectAdd( &src2, &src1, &rect );
	
	// ]ݒ
	drc1.x = drect ? drect->x : 0;
	drc1.y = drect ? drect->y : 0;
	drc1.w = src2.w;
	drc1.h = src2.h;
	RectAdd( &drc2, &drc1, dst->GetRect() );
	
	if( src2.w > drc2.w ) src2.w = drc2.w;
	if( src2.h > drc2.h ) src2.h = drc2.h;
	
	if( !src2.w || !src2.h || !drc2.w || !drc2.h ) return;
	
	BYTE *psrc = (BYTE *)pixels           + src2.y * pitch        + src2.x * bpp        / 8;
	BYTE *pdst = (BYTE *)dst->GetPixels() + drc2.y * dst->Pitch() + drc2.x * dst->Bpp() / 8;
	
	if( dst->Bpp() == bpp ){	// ]Ɠ]bppꍇ
		for( int i=0; i < src2.h; i++ ){
			memcpy( pdst, psrc, src2.w * bpp / 8 );
			psrc += pitch;
			pdst += dst->Pitch();
		}
	}else if( dst->Bpp() == 16 && bpp == 8 ){	// 8 -> 16 bpp (5:6:5)
		for( int i=0; i < src2.h; i++ ){
			for( int j=0; j < src2.w; j++ ){
				WORD wdat = (WORD)(palette->colors[*psrc].r & 0xf8) << 8
				          | (WORD)(palette->colors[*psrc].g & 0xfc) << 3
				          | (WORD)(palette->colors[*psrc].b & 0xf8) >> 3;
				*pdst++ = (BYTE)( wdat       & 0xff);
				*pdst++ = (BYTE)((wdat >> 8) & 0xff);
				psrc++;
			}
			psrc += pitch - src2.w;
			pdst += dst->Pitch() - src2.w * 2;
		}
	}else if( dst->Bpp() == 24 && bpp == 8 ){	// 8 -> 24 bpp
		for( int i=0; i < src2.h; i++ ){
			for( int j=0; j < src2.w; j++ ){
				*pdst++ = palette->colors[*psrc].b;
				*pdst++ = palette->colors[*psrc].g;
				*pdst++ = palette->colors[*psrc].r;
				psrc++;
			}
			psrc += pitch - src2.w;
			pdst += dst->Pitch() - src2.w * 3;
		}
	}
}

